#include "CPartsManager.h"



CPartsManager::CPartsManager()
{
	ItemCount = 0;
	DisableSorting = false;
}

void CPartsManager::RemoveAll()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		delete Items[Teller];
		Items[Teller] = NULL;
	}
	ItemCount=0;
}
void CPartsManager::Remove(int PlayFieldXin,int PlayFieldYin)
{
	int Teller1,Teller2;
	for (Teller1=0;Teller1<ItemCount;Teller1++)
	{
		if ((Items[Teller1]->GetLocation().x == PlayFieldXin) && (Items[Teller1]->GetLocation().y == PlayFieldYin))
		{
			delete Items[Teller1];
			for (Teller2=Teller1;Teller2<ItemCount-1;Teller2++)
				Items[Teller2] = Items[Teller2+1];
			ItemCount--;
			Teller1--;
		}
	}
}

void CPartsManager::Remove(int PlayFieldXin,int PlayFieldYin,int Type)
{
	int Teller1,Teller2;
	for (Teller1=0;Teller1<ItemCount;Teller1++)
	{
		if ((Items[Teller1]->GetLocation().x == PlayFieldXin) && (Items[Teller1]->GetLocation().y == PlayFieldYin) && (Items[Teller1]->GetId() == Type))
		{
			delete Items[Teller1];
			for (Teller2=Teller1;Teller2<ItemCount-1;Teller2++)
				Items[Teller2] = Items[Teller2+1];
			ItemCount--;
			Teller1--;
		}
	}
}

void CPartsManager::Add(CPart *Part)
{
	if( ItemCount < DMAXPARTS )
	{
	    Part->ParentList = this;
		Items[ItemCount] = Part;
		ItemCount++;
		Sort();
	}
}


void CPartsManager::Sort()
{
	int Teller1,Teller2,Index;
	CPart *Part;
	if (!DisableSorting)
	{
		for (Teller1 = 1; Teller1 <ItemCount;Teller1++)
		{
			Index = Items[Teller1]->GetZ();
			Part = Items[Teller1];
			Teller2 = Teller1;
			while ((Teller2 > 0) && (Items[Teller2-1]->GetZ() > Index))
			{
				Items[Teller2] = Items[Teller2 - 1];
				Teller2--;
			}
			Items[Teller2] = Part;
		}
	}

}

void CPartsManager::Save(char *Filename)
{
	FILE *Fp;
	int Teller,BufferPosition=0;
	char *Buffer;
	Buffer = new char[5*ItemCount];
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		Buffer[BufferPosition]= (char) Items[Teller]->GetId();
		Buffer[BufferPosition+1] = (char)Items[Teller]->GetLocation().x;
		Buffer[BufferPosition+2]= (char)Items[Teller]->GetLocation().y;
		Buffer[BufferPosition+3]= (char)Items[Teller]->GetLocation().w;
		Buffer[BufferPosition+4]= (char)Items[Teller]->GetLocation().h;
		BufferPosition += 5;
	}
	Fp = fopen(Filename,"wb");
	if (Fp)
	{
		fwrite(Buffer,1,5*ItemCount,Fp);
		fclose(Fp);
#ifdef GP2X
		sync();
#endif
	}
	delete[] Buffer;
}

void CPartsManager::Load(char *Filename)
{
	int X,Y,W,H,Type;
	FILE *Fp;
	Uint32 StartTime=0;
	int BufferPosition=0;
	long FileSize;
	char *Buffer;
	Fp = fopen(Filename,"rb");
	if(Fp)
	{
		RemoveAll();
		DisableSorting=true;
		StartTime = SDL_GetTicks();
		fseek (Fp , 0 , SEEK_END);
  		FileSize = ftell (Fp);
  		rewind (Fp);
		Buffer = new char[FileSize];
		fread(Buffer,1,FileSize,Fp);
		while(BufferPosition < FileSize)
		{
			Type = (int)Buffer[BufferPosition];
			X =(int)Buffer[BufferPosition+1];
			Y = (int)Buffer[BufferPosition+2];
			W = (int)Buffer[BufferPosition+3];
			H = (int)Buffer[BufferPosition+4];
			BufferPosition +=5;
			switch(Type)
			{
                //add type;
			}
		}
		delete[] Buffer;
		fclose(Fp);
		DisableSorting=false;
		Sort();
		//printf("Loading:%d\n",SDL_GetTicks() - StartTime);
	}

}

void CPartsManager::Move()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
		Items[Teller]->Move();
}

void CPartsManager::Draw(SDL_Surface *Surface)
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
        Items[Teller]->Draw(Surface);
	}
}


int CPartsManager::CollidePoint(int X,int Y)
{
    int Teller,Tmp=-1;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
        if((X >= Items[Teller]->GetLocation().x) && (X < Items[Teller]->GetLocation().x + Items[Teller]->GetLocation().w) &&
           (Y >= Items[Teller]->GetLocation().y) && (Y < Items[Teller]->GetLocation().y + Items[Teller]->GetLocation().h))
        {
            Tmp = Teller;
            break;
        }
	}
	return Tmp;
}

int CPartsManager::CollidePointRealLocation(int X,int Y)
{
    int Teller,Tmp=-1;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
        if((X >= Items[Teller]->GetRealLocation().x) && (X < Items[Teller]->GetRealLocation().x + Items[Teller]->GetRealLocation().w) &&
           (Y >= Items[Teller]->GetRealLocation().y) && (Y < Items[Teller]->GetRealLocation().y + Items[Teller]->GetRealLocation().h))
        {
            Tmp = Teller;
            break;
        }
	}
	return Tmp;
}

int CPartsManager::CollideRect(SDL_Rect Rect,int Index)
{
    int Teller,Tmp=-1;
    if((Index >=0) && (Index <ItemCount))
    {

        for (Teller=Index;Teller<ItemCount;Teller++)
        {
            //printf("portman : checking: (%d %d %d %d) - > (%d %d %d %d )",Items[Teller]->GetLocation().x,Items[Teller]->GetLocation().y ,Items[Teller]->GetLocation().x + Items[Teller]->GetLocation().w-1,Items[Teller]->GetLocation().y + Items[Teller]->GetLocation().h-1,Rect.x+Rect.w-1,Rect.y+Rect.h-1,Rect.x,Rect.y );
            if (!(Items[Teller]->GetLocation().x > Rect.x+Rect.w-1 || Items[Teller]->GetLocation().x + Items[Teller]->GetLocation().w-1 < Rect.x ||
                 Items[Teller]->GetLocation().y > Rect.y+Rect.h-1 || Items[Teller]->GetLocation().y + Items[Teller]->GetLocation().h-1 < Rect.y))
            {
                //printf("Correct\n");
                Tmp = Teller;
                break;
            }
           // printf("\n");
        }
    }
	return Tmp;
}

CPartsManager::~CPartsManager()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		delete Items[Teller];
		Items[Teller] = NULL;
	}
	//printf("Deleted worldparts\n");
}



