#ifndef CWHEEL_H
#define CWHEEL_H

#include <vector>
#include "CVector2D.h"
#include "C2dMatrix.h"
#include "math.h"
class CWheel
{
    private:
        CVector2D m_forwardAxis, m_sideAxis;
        float m_wheelTorque, m_wheelSpeed, m_wheelInertia, m_wheelRadius;
        CVector2D m_Position;

    public:

        CWheel(CVector2D position, float radius)
        {
            m_Position = position;
            SetSteeringAngle(0);
            m_wheelSpeed = 0;
            m_wheelRadius = radius;
            m_wheelInertia = radius * radius; //fake value
        }

        void SetSteeringAngle(float newAngle)
        {
            C2DMatrix mat;
            vector<CVector2D> Vector2ds;

            //foward CVector2D
            Vector2ds.push_back(CVector2D(0,1));
            Vector2ds.push_back(CVector2D(-1,0));


            mat.Rotate(newAngle / (float)M_PI * 180.0f);
            mat.TransformCVector2Ds(Vector2ds);

            m_forwardAxis =  CVector2D(Vector2ds[0].x, Vector2ds[0].y);
            m_sideAxis = CVector2D(Vector2ds[1].x, Vector2ds[1].y);
        }

        void AddTransmissionTorque(float newValue)
        {
            m_wheelTorque += newValue;
        }

        float GetWheelSpeed()
        {
            return m_wheelSpeed;
        }

        CVector2D GetAttachPoint()
        {
            return m_Position;
        }

        CVector2D CalculateForce(CVector2D relativeGroundSpeed, float timeStep)
        {
            //calculate speed of tire patch at ground
            CVector2D patchSpeed = -m_forwardAxis * m_wheelSpeed * m_wheelRadius;

            //get velocity difference between ground and patch
            CVector2D velDifference = relativeGroundSpeed + patchSpeed;

            //project ground speed onto side axis
            float forwardMag = 0;
            CVector2D sideVel = velDifference.Project(m_sideAxis);
            CVector2D forwardVel = velDifference.Project(m_forwardAxis, forwardMag);

            //calculate super fake friction forces
            //calculate response force
            CVector2D responseForce = -sideVel * 2.0f;
            responseForce -= forwardVel;

            //calculate torque on wheel
            m_wheelTorque += forwardMag * m_wheelRadius;

            //integrate total torque into wheel
            m_wheelSpeed += m_wheelTorque / m_wheelInertia * timeStep;

            //clear our transmission torque accumulator
            m_wheelTorque = 0;

            //return force acting on body
            return responseForce;
        }
};
#endif
