#include <stdlib.h>
#include <fstream>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <dirent.h>
#include <sys/stat.h>
#include "dynamate_base.h"
#include "dynamate_levels.h"
#include "defines.h"
#include "cursor.h"
#include "gp2x.h"
#include <SDL/SDL_rotozoom.h>
#include "SFont.h"
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_mixer.h>
#include "CInput.h"
#include <SDL/SDL_framerate.h>
#include "CUsbJoystickSetup.h"
#include "Functions.h"

#ifdef GP2X
#include <SDL/SDL_gp2x.h>
#endif


SDL_Surface *LevelData,
            *Screen,
            *Background,
            *BackgroundLevelEditor,
            *Menu,
            *LevelTitle,
            *IMGIntro1,
            *IMGIntro2,
            *IMGIntro3,
            *IMGIntro4,
//            *YouDiedTitle,
//            *SelectLevelTitle,
//            *GameMenuTitle,
//            *Quit1,
//            *Quit2,
//            *Retry1,
//            *Retry2,
//            *ResumePlay1,
//            *ResumePlay2,
//            *LastUnlocked1,
//            *LastUnlocked2,
//            *Next1,
//            *Next2,
//            *Previous1,
//            *Previous2,
            *Unlocked,
            *Locked,
//            *SelectLevel1,
//            *SelectLevel2,
//            *Play1,
//            *Play2,
//            *YouWonTitle,
            *PieceOutsidePlayfield,
            *InfiniteLoop,
            *DeathPiece,
            *OutOfMoves,
            *LevelCompleted,
            *AllLevelsCompleted,
//            *MainMenuTitle,
//            *UserLevels1,
//            *UserLevels2,
//            *StaticLevels1,
//            *StaticLevels2,
//            *LevelEditor1,
//            *LevelEditor2,
            *TitleScreen,
//            *Back1,
//            *Back2,
//            *New1,
//            *New2,
//            *Edit1,
//            *Edit2,
//            *Save1,
//            *Save2,
//            *LevelEditorTitle,
            *SFontDark,
            *SFontLight,
            *Buffer,
            *Buffer1;

SFont_Font *FontDark,*FontLight;
CCursor *Cursor;
Uint32 NextTime=0;
GameStates GameState=GSINTRO;
int HighScores[MAXSTATICLEVELS];
int lvl = 0,MaxLevels,UserLevelCount=0,MusicCount=0,SelectedMusic=0,Volume=128,StartScreenX=0,StartScreenY=0,InitialStartScreenX=20,InitialStartScreenY=24;
bool StaticLevels,SaveEnabled=false,GlobalSoundEnabled=true,TvOutMode=false;
TTF_Font* Font,*SmallFont;
SDL_Color Brown = {118,100,62,0},DarkBlue = {0,99,180,0} ,Color = {255,255,255,0};
char UnlockData[30];
dm_user_lev UserLevel;
char UserLevelsFilenames[MAXUSERLEVELS][FILENAME_MAX];
FPSmanager Fpsman;
int framecounter=0,fps=0,SCREENPOSMULTIPLY=1,WINDOW_WIDTH=320,WINDOW_HEIGHT=240,TileWidth=15,TileHeight=15;
Uint32 Time=0;
char TextFps[512];
Mix_Chunk *Sounds[NROFSOUNDS];
Mix_Music *Music[MAXMUSICFILES];
CUsbJoystickSetup *JoystickSetup;

// set the volume of the music and both channels (left/right)
void SetVolume(const int VolumeIn)
{
	Mix_Volume(0,VolumeIn);
	Mix_Volume(1,VolumeIn);
	Mix_VolumeMusic(VolumeIn);
}

// increase the volume with 8
void IncVolume()
{
	if (Volume < 128)
	{
		Volume += 8;
		SetVolume(Volume);
	}
}

// decrease the volume with 8
void DecVolume()
{
	if (Volume > 0)
	{
		Volume-= 8;
		SetVolume(Volume);
	}
}

void LoadHighScores()
{
 	FILE *File;
 	int Teller;
 	File = fopen("./high.dat","r");
 	if(File)
 	{
		for(Teller=0;Teller<MAXSTATICLEVELS;Teller++)
            fscanf(File,"%d\n",&HighScores[Teller]);
		fclose(File);
 	}
 	else
 	{
		for(Teller=0;Teller<MAXSTATICLEVELS;Teller++)
            HighScores[Teller] = 0;
	}
}

void SaveHighScores()
{
 	FILE *File;
 	int Teller;
 	File = fopen("./high.dat","w");
 	if(File)
 	{
		for(Teller=0;Teller<MAXSTATICLEVELS;Teller++)
            fprintf(File,"%d\n",HighScores[Teller]);
		fclose(File);
#if defined GP2X || defined CAANOO
		sync();
#endif
 	}
}

void LoadSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","r");
 	if(SettingsFile)
 	{
		fscanf(SettingsFile,"Volume=%d\n",&Volume);
		SetVolume(Volume);
		if(TvOutMode)
		{
            fscanf(SettingsFile,"StartScreenX=%d\n",&StartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&StartScreenY);
		}
		else
        {
            fscanf(SettingsFile,"StartScreenX=%d\n",&InitialStartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&InitialStartScreenY);
            StartScreenX = 0;
            StartScreenY = 0;
        }
		fclose(SettingsFile);
 	}
 	else
 	{
		SetVolume(128);
		StartScreenX = (Screen->w - Buffer->w) / 2;
        StartScreenY = (Screen->h - Buffer->h) / 2;
	}
}

void SaveSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","w");
 	if(SettingsFile)
 	{
		fprintf(SettingsFile,"Volume=%d\n",Volume);
        if(TvOutMode)
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",StartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",StartScreenY);
        }
        else
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",InitialStartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",InitialStartScreenY);
        }
		fclose(SettingsFile);
#if defined GP2X || defined CAANOO
		sync();
#endif
 	}
}

void SearchForMusic()
{
	struct dirent *Entry;
	DIR *Directory;
	struct stat Stats;
	int Teller;
	char FileName[FILENAME_MAX];
	if (GlobalSoundEnabled)
		Music[0] = Mix_LoadMUS("./music/title.mod");
	Teller=1;
	Directory = opendir("./music");
	if (Directory)
	{
		Entry=readdir(Directory);
		while(Entry)
		{
			sprintf(FileName,"./music/%s",Entry->d_name);
			stat(FileName,&Stats);
			if(!S_ISDIR(Stats.st_mode))
			{
				if(strncmp(".", Entry->d_name, 1) && (strcmp("title.mod",Entry->d_name) != 0) && (Teller< MAXMUSICFILES))
				{
				//	printf("%s\n",Entry->d_name);
					if (GlobalSoundEnabled)
					{
						Music[Teller] = Mix_LoadMUS(FileName);
						Teller++;
					}
				}
			}
			Entry=readdir(Directory);
		}
		closedir(Directory);
	}
	MusicCount = Teller;
}

void LoadSounds()
{
    if(GlobalSoundEnabled)
    {
		Sounds[SND_MENU] = Mix_LoadWAV("./sound/menu.wav");
		Sounds[SND_SELECT] = Mix_LoadWAV("./sound/select.wav");
		Sounds[SND_ERROR] = Mix_LoadWAV("./sound/error.wav");

		Sounds[SND_EXPLODE] = Mix_LoadWAV("./sound/explode.wav");
		Sounds[SND_MOVE] = Mix_LoadWAV("./sound/move.wav");
		Sounds[SND_TELEPORT] = Mix_LoadWAV("./sound/teleport.wav");
		Sounds[SND_LEVELPASS] = Mix_LoadWAV("./sound/passlevel.wav");
		Sounds[SND_LEVELFAIL] = Mix_LoadWAV("./sound/fail.wav");

    }
}

void UnLoadSounds()
{
    if (GlobalSoundEnabled)
    {
        if (!Mix_PlayingMusic())
            Mix_HaltMusic();

        int Teller;
        for (Teller=0;Teller<NROFSOUNDS;Teller++)
            if(Sounds[Teller])
                Mix_FreeChunk(Sounds[Teller]);

        for (Teller=0;Teller<MusicCount;Teller++)
            if(Music[Teller])
                Mix_FreeMusic(Music[Teller]);
    }

}


dm_u8 NextPiece(dm_u8 piece)
{
	switch (piece)
	{
		case DM_P_EMPTY				:return(0x80 | DM_P_VERT);
		//case S_GLUE3		:
		//case S_GLUE5		:
		case (0x80 | DM_P_VERT)		:return(0x80 | DM_P_HORI);
		case (0x80 | DM_P_HORI)		:return(0x80 | DM_P_TELE1);
		case (0x80 | DM_P_TELE1)	:return(0x80 | DM_P_TELE2);
		case (0x80 | DM_P_TELE2)	:return(0x80 | DM_P_ROTCOL);
		case (0x80 | DM_P_ROTCOL)	:return(0x80 | DM_P_ROT_CW);
		case (0x80 | DM_P_ROT_CW)	:return(0x80 | DM_P_ROT_CCW);
		case (0x80 | DM_P_ROT_CCW)	:return(0x80 | DM_P_GRAY);
		case (0x80 | DM_P_GRAY)		:return(0x80 | DM_P_DIE);
		case (0x80 | DM_P_DIE)		:return(0x80 | DM_P_BLACK);
		case (0x80 | DM_P_BLACK)	:return(0x80 | DM_P_YELLOW);
		case (0x80 | DM_P_YELLOW)	:return(0x80 | DM_P_RED);
		case (0x80 | DM_P_RED)		:return(0x80 | DM_P_GREEN);
		case (0x80 | DM_P_GREEN)	:return(0x80 | DM_P_BLUE);
		case (0x80 | DM_P_BLUE)		:return DM_P_VERT;

		//case M_GLUE3		:
		//case M_GLUE5		:
		case DM_P_VERT		:return DM_P_HORI;
		case DM_P_HORI		:return DM_P_TELE1;
		case DM_P_TELE1		:return DM_P_TELE2;
		case DM_P_TELE2		:return DM_P_ROTCOL;
		case DM_P_ROTCOL	:return DM_P_ROT_CW;
		case DM_P_ROT_CW	:return DM_P_ROT_CCW;
		case DM_P_ROT_CCW	:return DM_P_GRAY;
		case DM_P_GRAY		:return DM_P_DIE;
		case DM_P_DIE		:return DM_P_BLACK;
		case DM_P_BLACK		:return DM_P_YELLOW;
		case DM_P_YELLOW	:return DM_P_RED;
		case DM_P_RED		:return DM_P_GREEN;
		case DM_P_GREEN		:return DM_P_BLUE;
		case DM_P_BLUE		:return(0x80 | DM_P_VERT);

		//case S_GLUE0		:
		//case M_GLUE0		:

		default				:return DM_P_EMPTY;
	}
}

dm_u8 PreviousPiece(dm_u8 piece)
{

	dm_u8 prevPiece, newPiece = piece;

	do
	{
		prevPiece = newPiece;
		newPiece = NextPiece(newPiece);
	} while(newPiece != piece);

	return prevPiece;
}

void DrawLevel(SDL_Surface *Surface)
{
    int x,y,tmp,piecenr;
    SDL_Rect Src,Dst;
    for(y=0;y<TILEROWS;y++)
    {
        for(x=0;x<TILECOLUMNS;x++)
        {
            tmp = dm_field()[(y*TILEROWS)+x];
            switch(L7(tmp))
            {
                case DM_P_RED:
                    if(SOLID(tmp))
                        piecenr = 14;
                    else
                        piecenr = 30;
                    break;
                case DM_P_BLUE:
                     if(SOLID(tmp))
                        piecenr = 16;
                    else
                        piecenr = 32;
                    break;
                case DM_P_GREEN:
                     if(SOLID(tmp))
                        piecenr = 15;
                    else
                        piecenr = 31;
                    break;
                case DM_P_YELLOW:
                    if(SOLID(tmp))
                        piecenr = 13;
                    else
                        piecenr = 29;
                    break;

                case DM_P_BLACK:
                    if(SOLID(tmp))
                        piecenr = 12;
                    else
                        piecenr = 28;
                    break;

                case DM_P_HORI:
                    if(SOLID(tmp))
                        piecenr = 4;
                    else
                        piecenr = 20;
                    break;

                case DM_P_VERT:
                    if(SOLID(tmp))
                        piecenr = 3;
                    else
                        piecenr = 19;
                    break;

                case DM_P_TELE1:
                    if(SOLID(tmp))
                        piecenr = 5;
                    else
                        piecenr = 21;
                    break;

                case DM_P_TELE2:
                    if(SOLID(tmp))
                        piecenr = 6;
                    else
                        piecenr = 22;
                    break;

                case DM_P_ROTCOL:
                    if(SOLID(tmp))
                        piecenr = 7;
                    else
                        piecenr = 23;
                    break;

                case DM_P_ROT_CW:
                    if(SOLID(tmp))
                        piecenr = 8;
                    else
                        piecenr = 24;
                    break;

                case DM_P_ROT_CCW:
                    if(SOLID(tmp))
                        piecenr = 9;
                    else
                        piecenr = 25;
                    break;

                case DM_P_GRAY:
                    if(SOLID(tmp))
                        piecenr = 10;
                    else
                        piecenr = 26;
                    break;

                case DM_P_DIE:
                    if(SOLID(tmp))
                        piecenr = 11;
                    else
                        piecenr = 27;
                    break;

                default:
                    piecenr=0;
                    break;
            }
            if(piecenr != 0)
            {
                Src.x = 0;
                Src.y = piecenr*TileHeight;
                Src.w = TileWidth;
                Src.h = TileHeight;
                Dst.x = x * TileWidth;
                Dst.y = y * TileHeight;
                Dst.w = TileWidth;
                Dst.h = TileHeight;
                SDL_BlitSurface(LevelData,&Src,Surface,&Dst);
                //dirty hack to force synchronous blitting
                //SDL_LockSurface(Surface);
                //SDL_UnlockSurface(Surface);
            }

        }
    }
}

void LoadLevel(char *FileName)
{
    FILE *f;
    f = fopen(FileName,"rb");
    if(f)
    {
        fread(&UserLevel,sizeof(dm_user_lev),1,f);
        fclose(f);
#if defined GP2X || defined CAANOO
        sync();
#endif
    dm_init_level(UserLevel.field);
    }
}

void SaveLevel1(char *FileName)
{
    FILE *f;
    f = fopen(FileName,"wb");
    if(f)
    {
        fwrite(&UserLevel,sizeof(dm_user_lev),1,f);
        fclose(f);
#if defined GP2X || defined CAANOO
        sync();
#endif
    }
}

void SaveLevel(char *FileName)
{
    dm_user_lev tmp;
    FILE *f;
    memset(tmp.hs,0,256);
    memcpy(&tmp.field,dm_field(),256);
    sprintf(tmp.name,"%s",UserLevel.name);
    sprintf(tmp.hs,"%s","");
    f = fopen(FileName,"wb");
    if(f)
    {
        fwrite(&tmp,sizeof(dm_user_lev),1,f);
        fclose(f);
#if defined GP2X || defined CAANOO
        sync();
#endif
    }
}

bool GetName(char *Text)
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    bool end=false,canceled = true;
    SDL_Surface *Tmp,*Tmp1;
    dm_u8 TmpPiece = NextPiece(DM_P_EMPTY);
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst,Src;
    char Text1[512];
    int x=0,y=0,x1,y1,piecenr;
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(3);
    while (!end)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        if(size >= 1)
          Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && ( Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if( Input->KeyboardHeld[SDLK_ESCAPE] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)] )
            end = true;

        if(Input->KeyboardHeld[SDLK_RETURN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)])
        {
            end = true;
            canceled = false;
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] ||
                                Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_q] ||  Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)]))
        {
            if(strlen(Text) < 20)
                if(((y*11)+x) < 65)
                    sprintf(Text,"%s%c",Text,NameSymbols[(y*11)+x]);
                else
                    if(((y*11)+x) == 65)
                    {
                        if(strlen(Text) > 0)
                        {
                            memset(Text1,0,512);
                            strncpy(Text1,Text,strlen(Text)-1 );
                            Text1[strlen(Text)] = '\0';
                            sprintf(Text,"%s\0 ",Text1);
                        }
                    }
                    else
                        if((y ==6) && (x==0))
                        {
                            if((strlen(Text) > 0) && Text[0] != ' ')
                            {
                                if (GlobalSoundEnabled)
                                    Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                                end=true;
                                canceled = false;
                            }
                        }
                        else
                        {
                            if (GlobalSoundEnabled)
                                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                            end=true;
                        }
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_BACKSPACE] || Input->KeyboardHeld[SDLK_y] ||
                                Input->KeyboardHeld[SDLK_b] || Input->KeyboardHeld[SDLK_DELETE] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]))
        {
            if(strlen(Text) > 0)
            {
                memset(Text1,0,512);
                strncpy(Text1,Text,strlen(Text)-1 );
                Text1[strlen(Text)] = '\0';
                sprintf(Text,"%s\0 ",Text1);
                if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            }
            Input->Delay();
        }

        if((Input->Ready()) && ( Input->KeyboardHeld[SDLK_RIGHT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)]))
        {
            x++;
            if((y>=0) && (y<=5))
            {
                if(x > 10)
                    x = 0;
            }
            else
            {
                if(x > 1)
                    x = 0;
            }
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_LEFT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)]))
        {
            x--;
            if((y>=0) && (y<=5))
            {
                if(x < 0)
                    x = 10;
            }
            else
                if(x < 0)
                    x = 1;
            Input->Delay();
        }

        if((Input->Ready()) && ( Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            if(y == 6)
                x = 0;
            y--;
            if(y<0)
                y=6;
            if(y == 6)
                x = 0;
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            if(y==5)
                x = 0;
            y++;
            if(y>6)
            {
                y = 0;
                x = 0;
            }
            Input->Delay();
        }


        SDL_BlitSurface(BackgroundLevelEditor,NULL,Buffer,NULL);

            Tmp = zoomSurface(LevelTitle,1,1,0);
            sprintf(Text1,"Level name: %s",Text);
            WriteText(Tmp,Font,Text1,strlen(Text1),0,8*SCREENPOSMULTIPLY,0,DarkBlue,true);
            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y = 15*SCREENPOSMULTIPLY;
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);

            Tmp = zoomSurface(Menu, 1, 1,0);
            for(y1 = 0; y1<6 ; y1++)
            {
                for(x1=0;x1<11;x1++)
                {
                    if((y1*11)+x1 >= 65)
                        break;
                    sprintf(Text1,"%c",NameSymbols[(y1*11)+x1]);
                    WriteText(Tmp,Font,Text1,1,(7+(x1*14))*SCREENPOSMULTIPLY,(20+(y1*14))*SCREENPOSMULTIPLY,0,DarkBlue,false);

                }
                if((y1*11)+x1 >= 65)
                        break;
            }
            sprintf(Text1,"%s","<");
            WriteText(Tmp,Font,Text1,1,(7+(10*14))*SCREENPOSMULTIPLY,(20+(5*14))*SCREENPOSMULTIPLY,0,DarkBlue,false);

            sprintf(Text1,"%s","OK");
            WriteText(Tmp,Font,Text1,2,50*SCREENPOSMULTIPLY,(25+(6*14))*SCREENPOSMULTIPLY,0,DarkBlue,false);

            sprintf(Text1,"%s","CANCEL");
            WriteText(Tmp,Font,Text1,6,75*SCREENPOSMULTIPLY,(25+(6*14))*SCREENPOSMULTIPLY,0,DarkBlue,false);

            if(y < 6)
                boxRGBA(Tmp,(3+(x*14))*SCREENPOSMULTIPLY,(20+(y*14))*SCREENPOSMULTIPLY,(3+((x+1)*14))*SCREENPOSMULTIPLY,(20+((y+1)*14))*SCREENPOSMULTIPLY,255,255,255,128);
            else
                if(x == 0)
                    boxRGBA(Tmp,45*SCREENPOSMULTIPLY,(25+(y*14))*SCREENPOSMULTIPLY,70*SCREENPOSMULTIPLY,(25+((y+1)*14))*SCREENPOSMULTIPLY,255,255,255,128);
                else
                    boxRGBA(Tmp,70*SCREENPOSMULTIPLY,(25+(y*14))*SCREENPOSMULTIPLY,120*SCREENPOSMULTIPLY,(25+((y+1)*14))*SCREENPOSMULTIPLY,255,255,255,128);


            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y = (WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            SDL_FreeSurface(Tmp1);
            SDL_FreeSurface(Tmp);

        TmpPiece = NextPiece(DM_P_EMPTY);
        for(y1 = 0;y1<6;y1++)
        {
            for(x1=0;x1<5;x1++)
            {
                switch(L7(TmpPiece))
                {
                    case DM_P_RED:
                        if(SOLID(TmpPiece))
                            piecenr = 14;
                        else
                            piecenr = 30;
                        break;
                    case DM_P_BLUE:
                         if(SOLID(TmpPiece))
                            piecenr = 16;
                        else
                            piecenr = 32;
                        break;
                    case DM_P_GREEN:
                         if(SOLID(TmpPiece))
                            piecenr = 15;
                        else
                            piecenr = 31;
                        break;
                    case DM_P_YELLOW:
                        if(SOLID(TmpPiece))
                            piecenr = 13;
                        else
                            piecenr = 29;
                        break;

                    case DM_P_BLACK:
                        if(SOLID(TmpPiece))
                            piecenr = 12;
                        else
                            piecenr = 28;
                        break;

                    case DM_P_HORI:
                        if(SOLID(TmpPiece))
                            piecenr = 4;
                        else
                            piecenr = 20;
                        break;

                    case DM_P_VERT:
                        if(SOLID(TmpPiece))
                            piecenr = 3;
                        else
                            piecenr = 19;
                        break;

                    case DM_P_TELE1:
                        if(SOLID(TmpPiece))
                            piecenr = 5;
                        else
                            piecenr = 21;
                        break;

                    case DM_P_TELE2:
                        if(SOLID(TmpPiece))
                            piecenr = 6;
                        else
                            piecenr = 22;
                        break;

                    case DM_P_ROTCOL:
                        if(SOLID(TmpPiece))
                            piecenr = 7;
                        else
                            piecenr = 23;
                        break;

                    case DM_P_ROT_CW:
                        if(SOLID(TmpPiece))
                            piecenr = 8;
                        else
                            piecenr = 24;
                        break;

                    case DM_P_ROT_CCW:
                        if(SOLID(TmpPiece))
                            piecenr = 9;
                        else
                            piecenr = 25;
                        break;

                    case DM_P_GRAY:
                        if(SOLID(TmpPiece))
                            piecenr = 10;
                        else
                            piecenr = 26;
                        break;

                    case DM_P_DIE:
                        if(SOLID(TmpPiece))
                            piecenr = 11;
                        else
                            piecenr = 27;
                        break;

                    default:
                        piecenr=0;
                        break;
                }
                if(piecenr != 0)
                {
                    Src.x = 0;
                    Src.y = piecenr*TileHeight;
                    Src.w = TileWidth;
                    Src.h = TileHeight;
                    Dst.x = (x1 * TileWidth)+(243*SCREENPOSMULTIPLY);
                    Dst.y = (y1 * TileHeight)+(150*SCREENPOSMULTIPLY);
                    Dst.w = TileWidth;
                    Dst.h = TileHeight;
                    SDL_BlitSurface(LevelData,&Src,Buffer,&Dst);
                    //dirty hack to force synchronous blitting
                   // SDL_LockSurface(Buffer);
                    //SDL_UnlockSurface(Buffer);
                }

                TmpPiece = NextPiece(TmpPiece);
                if(((y1)*5)+(x1) >= 27)
                    break;
            }
            if(((y1)*5)+(x1) >= 27)
                    break;
        }
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;
    return !canceled;
}


void PrintMsg(char *Text,char *Title)
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    bool end=false;
    SDL_Surface *Tmp,*Tmp1;
    dm_u8 TmpPiece = NextPiece(DM_P_EMPTY);
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst,Src;
    int x1,y1,piecenr;
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(10);
    while (!end)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && ( Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->KeyboardHeld[SDLK_a]  || Input->KeyboardHeld[SDLK_q] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] ||  Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_L)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_R)])
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            end = true;
        }

        SDL_BlitSurface(BackgroundLevelEditor,NULL,Buffer,NULL);
        DrawLevel(Buffer);

            Tmp = zoomSurface(Menu, 1, 1,0);
            WriteText(Tmp,Font,Text,strlen(Text),0,30*SCREENPOSMULTIPLY,0,DarkBlue,true);
            SFont_WriteCenter(Tmp,FontDark,4,Title);
            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y = (WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);

        TmpPiece = NextPiece(DM_P_EMPTY);
        for(y1 = 0;y1<6;y1++)
        {
            for(x1=0;x1<5;x1++)
            {
                switch(L7(TmpPiece))
                {
                    case DM_P_RED:
                        if(SOLID(TmpPiece))
                            piecenr = 14;
                        else
                            piecenr = 30;
                        break;
                    case DM_P_BLUE:
                         if(SOLID(TmpPiece))
                            piecenr = 16;
                        else
                            piecenr = 32;
                        break;
                    case DM_P_GREEN:
                         if(SOLID(TmpPiece))
                            piecenr = 15;
                        else
                            piecenr = 31;
                        break;
                    case DM_P_YELLOW:
                        if(SOLID(TmpPiece))
                            piecenr = 13;
                        else
                            piecenr = 29;
                        break;

                    case DM_P_BLACK:
                        if(SOLID(TmpPiece))
                            piecenr = 12;
                        else
                            piecenr = 28;
                        break;

                    case DM_P_HORI:
                        if(SOLID(TmpPiece))
                            piecenr = 4;
                        else
                            piecenr = 20;
                        break;

                    case DM_P_VERT:
                        if(SOLID(TmpPiece))
                            piecenr = 3;
                        else
                            piecenr = 19;
                        break;

                    case DM_P_TELE1:
                        if(SOLID(TmpPiece))
                            piecenr = 5;
                        else
                            piecenr = 21;
                        break;

                    case DM_P_TELE2:
                        if(SOLID(TmpPiece))
                            piecenr = 6;
                        else
                            piecenr = 22;
                        break;

                    case DM_P_ROTCOL:
                        if(SOLID(TmpPiece))
                            piecenr = 7;
                        else
                            piecenr = 23;
                        break;

                    case DM_P_ROT_CW:
                        if(SOLID(TmpPiece))
                            piecenr = 8;
                        else
                            piecenr = 24;
                        break;

                    case DM_P_ROT_CCW:
                        if(SOLID(TmpPiece))
                            piecenr = 9;
                        else
                            piecenr = 25;
                        break;

                    case DM_P_GRAY:
                        if(SOLID(TmpPiece))
                            piecenr = 10;
                        else
                            piecenr = 26;
                        break;

                    case DM_P_DIE:
                        if(SOLID(TmpPiece))
                            piecenr = 11;
                        else
                            piecenr = 27;
                        break;

                    default:
                        piecenr=0;
                        break;
                }
                if(piecenr != 0)
                {
                    Src.x = 0;
                    Src.y = piecenr*TileHeight;
                    Src.w = TileWidth;
                    Src.h = TileHeight;
                    Dst.x = (x1 * TileWidth)+(243*SCREENPOSMULTIPLY);
                    Dst.y = (y1 * TileHeight)+(150*SCREENPOSMULTIPLY);
                    Dst.w = TileWidth;
                    Dst.h = TileHeight;
                    SDL_BlitSurface(LevelData,&Src,Buffer,&Dst);
                    //SDL_LockSurface(Buffer);
                    //SDL_UnlockSurface(Buffer);
                }

                TmpPiece = NextPiece(TmpPiece);
                if(((y1)*5)+(x1) >= 27)
                    break;
            }
            if(((y1)*5)+(x1) >= 27)
                    break;
        }
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}

void LevelSelectMenu()
{

    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    SDL_Surface *Tmp,*Tmp1;
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst;
    int Selected = 0;
    char Text[256];
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(10);
    if(StaticLevels)
        dm_init_level(dml_levels[lvl].field);
    else
    {
        LoadLevel(UserLevelsFilenames[lvl]);
        dm_init_level(UserLevel.field);
    }
    while (GameState == GSLEVELSELECTMENU)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }

        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if(Input->KeyboardHeld[SDLK_ESCAPE] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)])
        {
            GameState = GSTITLESCREEN;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            Mix_FadeOutMusic(500);
            while(Mix_PlayingMusic())
            {
                SDL_Delay(10);
            }
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Selected++;
            if (Selected > 3)
                Selected = 0;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            Selected--;
            if (Selected < 0)
                Selected = 3;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)]))
        {
            switch (Selected)
            {
                case 0: // Next
                    lvl++;
                    if(lvl > MaxLevels-1)
                        lvl = 0;
                    if(StaticLevels)
                        dm_init_level(dml_levels[lvl].field);
                    else
                    {
                        LoadLevel(UserLevelsFilenames[lvl]);
                        dm_init_level(UserLevel.field);
                    }
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    break;
                case 1: // previous
                    lvl--;
                    if(lvl < 0 )
                        lvl = MaxLevels-1;
                    if(StaticLevels)
                        dm_init_level(dml_levels[lvl].field);
                    else
                    {
                        LoadLevel(UserLevelsFilenames[lvl]);
                        dm_init_level(UserLevel.field);
                    }
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    break;
                case 2:
                    if(StaticLevels)
                    {
                        if(UnlockData[lvl] == '1')
                        {
                            GameState = GSGAME;
                            Cursor->SetXY(8,8);
                            if (GlobalSoundEnabled)
                                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                        }
                        else
                        {
                            //play error sound;
                            if (GlobalSoundEnabled)
                                Mix_PlayChannel(-1,Sounds[SND_ERROR],0);
                        }
                    }
                    else
                    {
                        GameState = GSGAME;
                        Cursor->SetXY(8,8);
                        if (GlobalSoundEnabled)
                            Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    }
                    break;
                case 3: //quit
                    GameState = GSTITLESCREEN;
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    Mix_FadeOutMusic(500);
                    while(Mix_PlayingMusic())
                    {
                        SDL_Delay(10);
                    }
                    break;

            }
            Input->Delay();
        }

        SDL_BlitSurface(Background,NULL,Buffer,NULL);
        DrawLevel(Buffer);

            Tmp = zoomSurface(LevelTitle,1,1,0);
            if(StaticLevels)
                sprintf(Text,"Level %d: %s",lvl+1,dml_levels[lvl].name);
            else
                sprintf(Text,"Level %d: %s",lvl+1,UserLevel.name);
            WriteText(Tmp,Font,Text,strlen(Text),0,8*SCREENPOSMULTIPLY,0,DarkBlue,true);
            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y = 15*SCREENPOSMULTIPLY;
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);

            Tmp = zoomSurface(Menu, 1, 1,0);

//            Dst.x = (Tmp->w/2) - (SelectLevelTitle->w/2);
//            Dst.y = 4;
//            Dst.w = SelectLevelTitle->w;
//            Dst.h = SelectLevelTitle->h;
//            SDL_BlitSurface(SelectLevelTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp,FontDark,4*SCREENPOSMULTIPLY,"Select Level");
            if(Selected ==0)
            {
//                Dst.x = (Tmp->w/2) - (Next1->w/2);
//                Dst.y = 24;
//                Dst.w = Next1->w;
//                Dst.h = Next1->h;
//                SDL_BlitSurface(Next1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,24*SCREENPOSMULTIPLY,"Next");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Next2->w/2);
//                Dst.y = 24;
//                Dst.w = Next2->w;
//                Dst.h = Next2->h;
//                SDL_BlitSurface(Next2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,24*SCREENPOSMULTIPLY,"Next");
            }

            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (Previous1->w/2);
//                Dst.y = 44;
//                Dst.w = Previous1->w;
//                Dst.h = Previous1->h;
//                SDL_BlitSurface(Previous1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,44*SCREENPOSMULTIPLY,"Previous");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Previous2->w/2);
//                Dst.y = 44;
//                Dst.w = Previous2->w;
//                Dst.h = Previous2->h;
//                SDL_BlitSurface(Previous2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,44*SCREENPOSMULTIPLY,"Previous");
            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (Play1->w/2);
//                Dst.y = 64;
//                Dst.w = Play1->w;
//                Dst.h = Play1->h;
//                SDL_BlitSurface(Play1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,64*SCREENPOSMULTIPLY,"Play");
            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Play2->w/2);
//                Dst.y = 64;
//                Dst.w = Play2->w;
//                Dst.h = Play2->h;
//                SDL_BlitSurface(Play2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,64*SCREENPOSMULTIPLY,"Play");
            }

            if(Selected == 3)
            {
//                Dst.x = (Tmp->w/2) - (Quit1->w/2);
//                Dst.y = 84;
//                Dst.w = Quit1->w;
//                Dst.h = Quit1->h;
//                SDL_BlitSurface(Quit1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,84*SCREENPOSMULTIPLY,"Quit");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Quit2->w/2);
//                Dst.y = 84;
//                Dst.w = Quit2->w;
//                Dst.h = Quit2->h;
//                SDL_BlitSurface(Quit2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,84*SCREENPOSMULTIPLY,"Quit");
            }
            if(StaticLevels)
                if(UnlockData[lvl]== '1')
                {
                    Dst.x = (Tmp->w/2) - (Unlocked->w/2);
                    Dst.y = 108*SCREENPOSMULTIPLY;
                    Dst.w = Unlocked->w;
                    Dst.h = Unlocked->h;
                    SDL_BlitSurface(Unlocked,NULL,Tmp,&Dst);
                }
                else
                {
                    Dst.x = (Tmp->w/2) - (Locked->w/2);
                    Dst.y = 108*SCREENPOSMULTIPLY;
                    Dst.w = Locked->w;
                    Dst.h = Locked->h;
                    SDL_BlitSurface(Locked,NULL,Tmp,&Dst);
                }

            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y =(WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);
        if(StaticLevels)
            sprintf(Text,"Level: %d\nRecord: %d\nMoves: %d\nStones: %d",lvl+1,HighScores[lvl],dm_movecount(),dm_stones_left());
        else
            sprintf(Text,"Level: %d\nMoves: %d\nStones: %d",lvl+1,dm_movecount(),dm_stones_left());
        WriteText(Buffer,Font,Text,strlen(Text),242*SCREENPOSMULTIPLY,182*SCREENPOSMULTIPLY,0,Brown,false);
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}

void WinMenu()
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    SDL_Surface *Tmp,*Tmp1;
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst;
    int Selected;
    char Text[512];
    if(lvl == MaxLevels - 1)
        Selected = 3;
    else
        Selected = 1;
    while (SDL_PollEvent(&Event));
    if (GlobalSoundEnabled)
        Mix_PlayChannel(-1,Sounds[SND_LEVELPASS],0);
    CInput *Input = new CInput(10);
    while (GameState == GSWINMENU)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }

        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Selected++;
            if(lvl == MaxLevels - 1)
            {
                if (Selected > 3)
                    Selected = 2;
            }
            else
            {
                if (Selected > 3)
                    Selected = 1;
            }
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
           Selected--;
            if(lvl == MaxLevels - 1)
            {
                if (Selected < 2)
                    Selected = 3;
            }
            else
            {
                if (Selected < 1)
                    Selected = 3;
            }
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)]  || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)])
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            switch (Selected)
            {
                case 1: // Next
                    lvl++;
                    if(StaticLevels)
                        dm_init_level(dml_levels[lvl].field);
                    else
                    {
                        LoadLevel(UserLevelsFilenames[lvl]);
                        dm_init_level(UserLevel.field);
                    }
                    GameState = GSGAME;
                    Cursor->SetXY(8,8);
                    break;
                case 2: //level select
                    GameState = GSLEVELSELECTMENU;
                    break;
                case 3: //quit
                    GameState = GSTITLESCREEN;
                    break;

            }
        }

        SDL_BlitSurface(Background,NULL,Buffer,NULL);
        DrawLevel(Buffer);



            Tmp = zoomSurface(Menu, 1, 1,0);
//            Dst.x = (Tmp->w/2) - (YouDiedTitle->w/2);
//            Dst.y = 4;
//            Dst.w = YouDiedTitle->w;
//            Dst.h = YouDiedTitle->h;
//            SDL_BlitSurface(YouWonTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp,FontDark,4*SCREENPOSMULTIPLY,"You Win!");

            if(lvl == MaxLevels - 1)
            {
                Dst.x = (Tmp->w/2) - (AllLevelsCompleted->w/2);
                Dst.y = 24*SCREENPOSMULTIPLY;
                Dst.w = AllLevelsCompleted->w;
                Dst.h = AllLevelsCompleted->h;
                SDL_BlitSurface(AllLevelsCompleted,NULL,Tmp,&Dst);
            }
            else
            {
                Dst.x = (Tmp->w/2) - (LevelCompleted->w/2);
                Dst.y = 24*SCREENPOSMULTIPLY;
                Dst.w = LevelCompleted->w;
                Dst.h = LevelCompleted->h;
                SDL_BlitSurface(LevelCompleted,NULL,Tmp,&Dst);
            }

            if(lvl < MaxLevels - 1)
            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (Next1->w/2);
//                Dst.y = 59;
//                Dst.w = Next1->w;
//                Dst.h = Next1->h;
//                SDL_BlitSurface(Next1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,59*SCREENPOSMULTIPLY,"Next");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Next2->w/2);
//                Dst.y = 59;
//                Dst.w = Next2->w;
//                Dst.h = Next2->h;
//                SDL_BlitSurface(Next2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,59*SCREENPOSMULTIPLY,"Next");

            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel1->w/2);
//                Dst.y = 82;
//                Dst.w = SelectLevel1->w;
//                Dst.h = SelectLevel1->h;
//                SDL_BlitSurface(SelectLevel1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,82*SCREENPOSMULTIPLY,"SelectLevel");


            }
            else
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel2->w/2);
//                Dst.y = 82;
//                Dst.w = SelectLevel2->w;
//                Dst.h = SelectLevel2->h;
//                SDL_BlitSurface(SelectLevel2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,82*SCREENPOSMULTIPLY,"SelectLevel");

            }

            if(Selected == 3)
            {
//                Dst.x = (Tmp->w/2) - (Quit1->w/2);
//                Dst.y = 105;
//                Dst.w = Quit1->w;
//                Dst.h = Quit1->h;
//                SDL_BlitSurface(Quit1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,105*SCREENPOSMULTIPLY,"Quit");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Quit2->w/2);
//                Dst.y = 105;
//                Dst.w = Quit2->w;
//                Dst.h = Quit2->h;
//                SDL_BlitSurface(Quit2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,105*SCREENPOSMULTIPLY,"Quit");
            }

            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y =(WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);
        if(StaticLevels)
            sprintf(Text,"Level: %d\nRecord: %d\nMoves: %d\nStones: %d",lvl+1,HighScores[lvl],dm_movecount(),dm_stones_left());
        else
            sprintf(Text,"Level: %d\nMoves: %d\nStones: %d",lvl+1,dm_movecount(),dm_stones_left());
        WriteText(Buffer,Font,Text,strlen(Text),242*SCREENPOSMULTIPLY,182*SCREENPOSMULTIPLY,0,Brown,false);
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}



void DiedMenu()
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    char Text[256];
    SDL_Surface *Tmp,*Tmp1;
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst;
    int Selected = 1;
    while (SDL_PollEvent(&Event));
    if (GlobalSoundEnabled)
        Mix_PlayChannel(-1,Sounds[SND_LEVELFAIL],0);
    CInput *Input = new CInput(10);
    while (GameState == GSDIEDMENU)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }

        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Selected++;
            if (Selected > 3)
                Selected = 1;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            Selected--;
            if (Selected < 1)
             Selected = 3;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] ||  Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)])
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            switch (Selected)
            {
                case 1: // retry
                    if(StaticLevels)
                        dm_init_level(dml_levels[lvl].field);
                    else
                    {
                        LoadLevel(UserLevelsFilenames[lvl]);
                        dm_init_level(UserLevel.field);
                    }
                    GameState = GSGAME;
                    Cursor->SetXY(8,8);
                    break;
                case 2: //level select
                    GameState = GSLEVELSELECTMENU;
                    break;
                case 3: //quit
                    GameState = GSTITLESCREEN;
                    break;

            }
        }

        SDL_BlitSurface(Background,NULL,Buffer,NULL);
        DrawLevel(Buffer);


            Tmp = zoomSurface(Menu, 1, 1,0);
//            Dst.x = (Tmp->w/2) - (YouDiedTitle->w/2);
//            Dst.y = 4;
//            Dst.w = YouDiedTitle->w;
//            Dst.h = YouDiedTitle->h;
//            SDL_BlitSurface(YouDiedTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp,FontDark,4*SCREENPOSMULTIPLY,"You Died!");
            switch(dm_state())
            {
                case DM_OUT_OF_MOVES:
                    Dst.x = (Tmp->w/2) - (OutOfMoves->w/2);
                    Dst.y = 24*SCREENPOSMULTIPLY;
                    Dst.w = OutOfMoves->w;
                    Dst.h = OutOfMoves->h;
                    SDL_BlitSurface(OutOfMoves,NULL,Tmp,&Dst);
                    break;
                case DM_HIT_DIE_PIECE:
                    Dst.x = (Tmp->w/2) - (DeathPiece->w/2);
                    Dst.y = 24*SCREENPOSMULTIPLY;
                    Dst.w = DeathPiece->w;
                    Dst.h = DeathPiece->h;
                    SDL_BlitSurface(DeathPiece,NULL,Tmp,&Dst);
                    break;
                case DM_OUT_OF_FIELD:
                    Dst.x = (Tmp->w/2) - (PieceOutsidePlayfield->w/2);
                    Dst.y = 24*SCREENPOSMULTIPLY;
                    Dst.w = PieceOutsidePlayfield->w;
                    Dst.h = PieceOutsidePlayfield->h;
                    SDL_BlitSurface(PieceOutsidePlayfield,NULL,Tmp,&Dst);
                    break;
                case DM_INFINITE_LOOP:
                    Dst.x = (Tmp->w/2) - (InfiniteLoop->w/2);
                    Dst.y = 24*SCREENPOSMULTIPLY;
                    Dst.w = InfiniteLoop->w;
                    Dst.h = InfiniteLoop->h;
                    SDL_BlitSurface(InfiniteLoop,NULL,Tmp,&Dst);
                    break;
                default:
                    break;
            }

            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (Retry1->w/2);
//                Dst.y = 59;
//                Dst.w = Retry1->w;
//                Dst.h = Retry1->h;
//                SDL_BlitSurface(Retry1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,59*SCREENPOSMULTIPLY,"Retry");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Retry2->w/2);
//                Dst.y = 59;
//                Dst.w = Retry2->w;
//                Dst.h = Retry2->h;
//                SDL_BlitSurface(Retry2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,59*SCREENPOSMULTIPLY,"Retry");
            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel1->w/2);
//                Dst.y = 82;
//                Dst.w = SelectLevel1->w;
//                Dst.h = SelectLevel1->h;
//                SDL_BlitSurface(SelectLevel1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,82*SCREENPOSMULTIPLY,"Select Level");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel2->w/2);
//                Dst.y = 82;
//                Dst.w = SelectLevel2->w;
//                Dst.h = SelectLevel2->h;
//                SDL_BlitSurface(SelectLevel2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,82*SCREENPOSMULTIPLY,"Select Level");
            }

            if(Selected == 3)
            {
//                Dst.x = (Tmp->w/2) - (Quit1->w/2);
//                Dst.y = 105;
//                Dst.w = Quit1->w;
//                Dst.h = Quit1->h;
//                SDL_BlitSurface(Quit1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,105*SCREENPOSMULTIPLY,"Quit");
            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Quit2->w/2);
//                Dst.y = 105;
//                Dst.w = Quit2->w;
//                Dst.h = Quit2->h;
//                SDL_BlitSurface(Quit2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,105*SCREENPOSMULTIPLY,"Quit");
            }

            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y =(WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);
        if(StaticLevels)
            sprintf(Text,"Level: %d\nRecord: %d\nMoves: %d\nStones: %d",lvl+1,HighScores[lvl],dm_movecount(),dm_stones_left());
        else
            sprintf(Text,"Level: %d\nMoves: %d\nStones: %d",lvl+1,dm_movecount(),dm_stones_left());
        WriteText(Buffer,Font,Text,strlen(Text),242*SCREENPOSMULTIPLY,182*SCREENPOSMULTIPLY,0,Brown,false);
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
       //SDL_Delay(WaitForFrame());
       SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}

void IngameMenu()
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    SDL_Surface *Tmp,*Tmp1;
    char Text[512];
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst;
    int Selected = 0;
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(10);
    while (GameState == GSINGAMEMENU)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }

        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && ( Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if(Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)] || Input->KeyboardHeld[SDLK_ESCAPE])
        {
            GameState = GSGAME;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Selected++;
            if (Selected > 3)
                Selected = 0;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            Selected--;
            if (Selected < 0)
                Selected = 3;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if(Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)])
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            switch (Selected)
            {
                case 0: // Resume play
                    GameState = GSGAME;
                    break;
                case 1: // retry
                    if(StaticLevels)
                        dm_init_level(dml_levels[lvl].field);
                    else
                    {
                        LoadLevel(UserLevelsFilenames[lvl]);
                        dm_init_level(UserLevel.field);
                    }
                    Cursor->SetXY(8,8);
                    GameState = GSGAME;
                    break;
                case 2: //level select
                    GameState = GSLEVELSELECTMENU;
                    break;
                case 3: //quit
                    Mix_FadeOutMusic(500);
                    while(Mix_PlayingMusic())
                    {
                        SDL_Delay(10);
                    }
                    GameState = GSTITLESCREEN;
                    break;

            }
        }

        SDL_BlitSurface(Background,NULL,Buffer,NULL);
        DrawLevel(Buffer);


            Tmp = zoomSurface(Menu, 1, 1,0);
//            Dst.x = (Tmp->w/2) - (GameMenuTitle->w/2);
//            Dst.y = 4;
//            Dst.w = GameMenuTitle->w;
//            Dst.h = GameMenuTitle->h;
//            SDL_BlitSurface(GameMenuTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp,FontDark,4*SCREENPOSMULTIPLY,"Game Menu");
            if(Selected == 0)
            {
//                Dst.x = (Tmp->w/2) - (ResumePlay1->w/2);
//                Dst.y = 27;
//                Dst.w = ResumePlay1->w;
//                Dst.h = ResumePlay1->h;
//                SDL_BlitSurface(ResumePlay1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,27*SCREENPOSMULTIPLY,"Resume Play");
            }
            else
            {
//                Dst.x = (Tmp->w/2) - (ResumePlay2->w/2);
//                Dst.y = 27;
//                Dst.w = ResumePlay2->w;
//                Dst.h = ResumePlay2->h;
//                SDL_BlitSurface(ResumePlay2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,27*SCREENPOSMULTIPLY,"Resume Play");
            }

            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (Retry1->w/2);
//                Dst.y = 50;
//                Dst.w = Retry1->w;
//                Dst.h = Retry1->h;
//                SDL_BlitSurface(Retry1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,50*SCREENPOSMULTIPLY,"Retry");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Retry2->w/2);
//                Dst.y = 50;
//                Dst.w = Retry2->w;
//                Dst.h = Retry2->h;
//                SDL_BlitSurface(Retry2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,50*SCREENPOSMULTIPLY,"Retry");
            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel1->w/2);
//                Dst.y = 73;
//                Dst.w = SelectLevel1->w;
//                Dst.h = SelectLevel1->h;
//                SDL_BlitSurface(SelectLevel1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,73*SCREENPOSMULTIPLY,"Select Level");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel2->w/2);
//                Dst.y = 73;
//                Dst.w = SelectLevel2->w;
//                Dst.h = SelectLevel2->h;
//                SDL_BlitSurface(SelectLevel2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,73*SCREENPOSMULTIPLY,"Select Level");
            }

            if(Selected == 3)
            {
//                Dst.x = (Tmp->w/2) - (Quit1->w/2);
//                Dst.y = 96;
//                Dst.w = Quit1->w;
//                Dst.h = Quit1->h;
//                SDL_BlitSurface(Quit1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,96*SCREENPOSMULTIPLY,"Quit");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Quit2->w/2);
//                Dst.y = 96;
//                Dst.w = Quit2->w;
//                Dst.h = Quit2->h;
//                SDL_BlitSurface(Quit2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,96*SCREENPOSMULTIPLY,"Quit");
            }

           if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y =(WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);
        if(StaticLevels)
            sprintf(Text,"Level: %d\nRecord: %d\nMoves: %d\nStones: %d",lvl+1,HighScores[lvl],dm_movecount(),dm_stones_left());
        else
            sprintf(Text,"Level: %d\nMoves: %d\nStones: %d",lvl+1,dm_movecount(),dm_stones_left());
        WriteText(Buffer,Font,Text,strlen(Text),242*SCREENPOSMULTIPLY,182*SCREENPOSMULTIPLY,0,Brown,false);
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}

int GetLastUnLocked()
{
    if(!StaticLevels)
        return 0;

    int teller = 0;
    while(UnlockData[teller] == '1')
    {
      teller++;
    }
    return teller-1;
}

void LevelTypeMenu(void)
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    SDL_Surface *Tmp,*Tmp1;
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst;
    int Selected = 0;
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(10);
    while (GameState == GSLEVELTYPESELECT)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        if(size >= 1)
           Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if(Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)] || Input->KeyboardHeld[SDLK_ESCAPE])
        {
            GameState = GSTITLESCREEN;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            if(UserLevelCount > 0)
            {
                Selected++;
                if (Selected > 2)
                    Selected = 0;
            }
            else
            {
                if(Selected == 2)
                    Selected = 0;
                else
                    if(Selected == 0)
                        Selected = 2;
            }
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            if(UserLevelCount > 0)
            {
                Selected--;
                if (Selected < 0)
                    Selected = 2;
            }
            else
            {
                if(Selected == 2)
                    Selected = 0;
                else
                    if(Selected == 0)
                        Selected = 2;
            }
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if(Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] ||  Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)])
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            switch (Selected)
            {
                case 0: // static
                    StaticLevels=true;
                    //set to last unlocked level!
                    lvl = GetLastUnLocked();
                    dm_init_level(dml_levels[lvl].field);
                    MaxLevels = MAXSTATICLEVELS;
                    GameState = GSGAME;
                    Cursor->SetXY(8,8);
                    Mix_FadeOutMusic(500);
                    while(Mix_PlayingMusic())
                    {
                        SDL_Delay(10);
                    }
                    break;
                case 1: // user
                    StaticLevels = false;
                    lvl = 0;
                    MaxLevels = UserLevelCount;
                    LoadLevel(UserLevelsFilenames[lvl]);
                    dm_init_level(UserLevel.field);
                    GameState = GSLEVELSELECTMENU;
                    Mix_FadeOutMusic(500);
                    while(Mix_PlayingMusic())
                    {
                        SDL_Delay(10);
                    }
                    break;
                case 2: // back
                    GameState = GSTITLESCREEN;
                    break;

            }
        }

        SDL_BlitSurface(TitleScreen,NULL,Buffer,NULL);


            Tmp = zoomSurface(Menu, 1, 1,0);
//            Dst.x = (Tmp->w/2)  - (SelectLevelTitle->w/2);
//            Dst.y = 4;
//            Dst.w = SelectLevelTitle->w;
//            Dst.h = SelectLevelTitle->h;
//            SDL_BlitSurface(SelectLevelTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp,FontDark,3*SCREENPOSMULTIPLY,"Leveltype");
            if(Selected == 0)
            {
//                Dst.x = (Tmp->w/2) - (StaticLevels1->w/2);
//                Dst.y = 27;
//                Dst.w = StaticLevels1->w;
//                Dst.h = StaticLevels1->h;
//                SDL_BlitSurface(StaticLevels1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,27*SCREENPOSMULTIPLY,"Static Levels");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (StaticLevels2->w/2);
//                Dst.y = 27;
//                Dst.w = StaticLevels2->w;
//                Dst.h = StaticLevels2->h;
//                SDL_BlitSurface(StaticLevels2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,27*SCREENPOSMULTIPLY,"Static Levels");
            }

            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (UserLevels1->w/2);
//                Dst.y = 50;
//                Dst.w = UserLevels1->w;
//                Dst.h = UserLevels1->h;
//                SDL_BlitSurface(UserLevels1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,50*SCREENPOSMULTIPLY,"User Levels");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (UserLevels2->w/2);
//                Dst.y = 50;
//                Dst.w = UserLevels2->w;
//                Dst.h = UserLevels2->h;
//                SDL_BlitSurface(UserLevels2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,50*SCREENPOSMULTIPLY,"User Levels");
            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (Back1->w/2);
//                Dst.y = 73;
//                Dst.w = Back1->w;
//                Dst.h = Back1->h;
//                SDL_BlitSurface(Back1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,73*SCREENPOSMULTIPLY,"Back");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Back2->w/2);
//                Dst.y = 73;
//                Dst.w = Back2->w;
//                Dst.h = Back2->h;
//                SDL_BlitSurface(Back2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,73*SCREENPOSMULTIPLY,"Back");
            }

            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = (WINDOW_WIDTH/2) -(Tmp1->w / 2);
            Dst.y = 150*SCREENPOSMULTIPLY -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);

#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);

    }
    delete Input;
}

void TitleScreenMenu(void)
{
    SDL_Rect Rect;
    SDL_Surface *Tmp=NULL,*Tmp1=NULL;
    float size=0.01;
    SDL_Rect Dst;
    int Selected = 0;
    CInput *Input = new CInput(10);
    while (GameState == GSTITLESCREEN)
    {
        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)])
            GameState = GSQUIT;
        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]) &&
           !(Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r]))
        {
            Selected++;
            if (Selected > 3)
                Selected = 0;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]) &&
           !(Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r]))
        {
            Selected--;
            if (Selected < 0)
                Selected = 3;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)])
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            switch (Selected)
            {
                case 0: // play
                    GameState = GSLEVELTYPESELECT;
                    break;
                case 1: // Level editor
                    memset(UserLevel.name,0,256);
                    memset(UserLevel.hs,0,256);
                    memset(UserLevel.field,0,256);
                    dm_init_level(UserLevel.field);
                    SaveEnabled = false;
                    StaticLevels = false;
                    Cursor->SetXY(8,8);
                    GameState = GSLEVELEDITORMENU;
                    break;
                case 2: // setup joystick
                    GameState = GSSETUPUSBJOYSTICKBUTTONS;
                    break;
                case 3: //Quit
                    GameState = GSQUIT;
                    break;
            }
        }

        if(Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r])
        {
            if(Input->Ready() && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
            {
                if(StartScreenX - 2 >= 0)
                    StartScreenX -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
            {
                if(StartScreenX + 2 + Buffer->w <= Screen->w)
                    StartScreenX +=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
            {
                if(StartScreenY - 2 >=0)
                    StartScreenY -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
            {
                if(StartScreenY + 2 + Buffer->h <= Screen->h)
                    StartScreenY +=2;
                Input->Delay();
            }
        }

        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        SDL_BlitSurface(TitleScreen,NULL,Buffer,NULL);


            Tmp = zoomSurface(Menu, 1, 1,0);
//            Dst.x = (Tmp->w/2)  - (MainMenuTitle->w/2);
//            Dst.y = 4;
//            Dst.w = MainMenuTitle->w;
//            Dst.h = MainMenuTitle->h;
//            //SDL_BlitSurface(MainMenuTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp, FontDark, 4*SCREENPOSMULTIPLY,"Main Menu");
            if(Selected == 0)
            {
//                Dst.x = (Tmp->w/2) - (Play1->w/2);
//                Dst.y = 27;
//                Dst.w = Play1->w;
//                Dst.h = Play1->h;
//                SDL_BlitSurface(Play1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontDark, 27*SCREENPOSMULTIPLY,"Play");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Play2->w/2);
//                Dst.y = 27;
//                Dst.w = Play2->w;
//                Dst.h = Play2->h;
//                SDL_BlitSurface(Play2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontLight, 27*SCREENPOSMULTIPLY,"Play");
            }

            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (LevelEditor1->w/2);
//                Dst.y = 50;
//                Dst.w = LevelEditor1->w;
//                Dst.h = LevelEditor1->h;
//                SDL_BlitSurface(LevelEditor1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontDark, 50*SCREENPOSMULTIPLY,"Level Editor");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (LevelEditor2->w/2);
//                Dst.y = 50;
//                Dst.w = LevelEditor2->w;
//                Dst.h = LevelEditor2->h;
//                SDL_BlitSurface(LevelEditor2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontLight, 50*SCREENPOSMULTIPLY,"Level Editor");
            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (Quit1->w/2);
//                Dst.y = 73;
//                Dst.w = Quit1->w;
//                Dst.h = Quit1->h;
//                SDL_BlitSurface(Quit1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontDark, 73*SCREENPOSMULTIPLY,"Joystick Setup");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Quit2->w/2);
//                Dst.y = 73;
//                Dst.w = Quit2->w;
//                Dst.h = Quit2->h;
//                SDL_BlitSurface(Quit2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontLight,73*SCREENPOSMULTIPLY,"Joystick Setup");
            }

            if(Selected == 3)
            {
//                Dst.x = (Tmp->w/2) - (Quit1->w/2);
//                Dst.y = 73;
//                Dst.w = Quit1->w;
//                Dst.h = Quit1->h;
//                SDL_BlitSurface(Quit1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontDark, 96*SCREENPOSMULTIPLY,"Quit");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Quit2->w/2);
//                Dst.y = 73;
//                Dst.w = Quit2->w;
//                Dst.h = Quit2->h;
//                SDL_BlitSurface(Quit2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp, FontLight,96*SCREENPOSMULTIPLY,"Quit");
            }

            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = (WINDOW_WIDTH/2) -(Tmp1->w / 2);
            Dst.y = 150*SCREENPOSMULTIPLY -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            SDL_FreeSurface(Tmp1);
            SDL_FreeSurface(Tmp);


#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
       SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}

void MovePiece(int x, int y,dm_dir  dir) {
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    dm_init_step(x,y,dir);
    char Text[256];
    while (dm_step() == 0)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }

       if(dm_cmd() != DM_NOTHING)
       {
            switch(dm_cmd())
            {
                case DM_MOVE:
                    //playsound
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_MOVE],0);
                    break;

                case DM_EXPL:
                    //playsound
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_EXPLODE],0);
                    break;

                case DM_TELE:
                    //playsound
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_TELEPORT],0);
                    break;

                case DM_NOTHING:
                default:
                    break;
            }
            SDL_BlitSurface(Background,NULL,Buffer,NULL);
            DrawLevel(Buffer);
            if(StaticLevels)
                sprintf(Text,"Level: %d\nRecord: %d\nMoves: %d\nStones: %d",lvl+1,HighScores[lvl],dm_movecount(),dm_stones_left());
            else
                sprintf(Text,"Level: %d\nMoves: %d\nStones: %d",lvl+1,dm_movecount(),dm_stones_left());
            WriteText(Buffer,Font,Text,strlen(Text),242*SCREENPOSMULTIPLY,182*SCREENPOSMULTIPLY,0,Brown,false);
            SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
            SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
            SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
            SDL_Flip(Screen);

       }
       SDL_Delay(50);
   }

}

void SearchForLevels()
{
	struct dirent *Entry;
	DIR *Directory;
	struct stat Stats;
	int Teller;
	char FileName[FILENAME_MAX];
	Teller = 0;
    Directory = opendir("./userlevels");
	if (Directory)
	{
		Entry=readdir(Directory);
		while(Entry)
		{
			sprintf(FileName,"./userlevels/%s",Entry->d_name);
			stat(FileName,&Stats);
			if(!S_ISDIR(Stats.st_mode))
			{
				if(strncmp(".", Entry->d_name, 1) && (Teller< MAXUSERLEVELS))
				{
                    sprintf(UserLevelsFilenames[Teller],"./userlevels/%s",Entry->d_name);
                    Teller++;
				}
			}
			Entry=readdir(Directory);
		}
		closedir(Directory);
	}
	UserLevelCount = Teller;
}


void LoadUnlockData(void)
{
    FILE *f;
    f = fopen("./dynamate.dat","rb");
    if(f)
    {
        fread(&UnlockData,sizeof(UnlockData),1,f);
        fclose(f);
    }
    else
    {
        for (int teller = 0;teller< MAXSTATICLEVELS;teller++)
            UnlockData[teller] = '0';
        UnlockData[0]= '1';
    }
}

void SaveUnlockData(void)
{
    FILE *f;
    f = fopen("./dynamate.dat","wb");
    fwrite(&UnlockData,sizeof(UnlockData),1,f);
    fclose(f);
#if defined GP2X || defined CAANOO
    sync();
#endif
}



void Game (void)
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    int prevx=0,prevy=0;
    SDL_Event Event;
    char Text[256];
    prevx = Cursor->GetX();
    prevy = Cursor->GetY();
    while (SDL_PollEvent(&Event));
    CInput *Input  = new CInput(5);

    while (GameState == GSGAME)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }

        Input->Update();

        if ((Input->Ready()) && ( Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if((Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r] || Input->KeyboardHeld[SDLK_PAGEUP]))
        {
            if (MusicCount > 1)
            {
                SelectedMusic++;
                if (SelectedMusic >= MusicCount)
                    SelectedMusic = 1;
                if(GlobalSoundEnabled)
                {
                    Mix_HaltMusic();
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    //Mix_HookMusicFinished(MusicFinished);
                    SetVolume(Volume);
                }
            }
            Input->Reset();
        }


        if(Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)] || Input->KeyboardHeld[SDLK_ESCAPE])
        {
            GameState = GSINGAMEMENU;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)]) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_UP);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a]  || Input->KeyboardHeld[SDLK_q] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)]) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_DOWN);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)]) && (Input->KeyboardHeld[SDLK_LEFT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_LEFT);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)]) && (Input->KeyboardHeld[SDLK_RIGHT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_RIGHT);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_UP);
            Cursor->SetXY(dm_srcx(),dm_srcy());
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_DOWN);
            Cursor->SetXY(dm_srcx(),dm_srcy());
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) && (Input->KeyboardHeld[SDLK_LEFT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_LEFT);
            Cursor->SetXY(dm_srcx(),dm_srcy());
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) && (Input->KeyboardHeld[SDLK_RIGHT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)]))
        {
            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_RIGHT);
            Cursor->SetXY(dm_srcx(),dm_srcy());
            Input->Delay();
        }


        //diagonals
        if((Input->Ready()) && !(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) &&
            (((Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]) &&
            (Input->KeyboardHeld[SDLK_LEFT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)]) )))
        {
            Cursor->SetY(Cursor->GetY()-1);
            Cursor->SetX(Cursor->GetX()-1);
            Input->Delay();
        }

        if((Input->Ready()) && !(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)])&&
            (((Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]) &&
            (Input->KeyboardHeld[SDLK_RIGHT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)]) ) ))
        {
            Cursor->SetY(Cursor->GetY()-1);
            Cursor->SetX(Cursor->GetX()+1);
            Input->Delay();
        }

        if((Input->Ready()) && !(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) &&
            ((( Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]) &&
            ( Input->KeyboardHeld[SDLK_LEFT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)]) )))
        {
            Cursor->SetY(Cursor->GetY()+1);
            Cursor->SetX(Cursor->GetX()-1);
            Input->Delay();
        }

        if((Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) &&
            (((Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]) &&
            (Input->KeyboardHeld[SDLK_RIGHT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)]) )))
        {
            Cursor->SetY(Cursor->GetY()+1);
            Cursor->SetX(Cursor->GetX()+1);
            Input->Delay();
        }


        if((Input->Ready()) && !(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            Cursor->SetY(Cursor->GetY()-1);
            Input->Delay();
        }

        if((Input->Ready()) && !(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Cursor->SetY(Cursor->GetY()+1);
            Input->Delay();
        }

        if((Input->Ready()) && !(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) &&(Input->KeyboardHeld[SDLK_LEFT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)]))
        {
            Cursor->SetX(Cursor->GetX()-1);
            Input->Delay();
        }

        if((Input->Ready()) && !( Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
            Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_b] ||
            Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)]) && (Input->KeyboardHeld[SDLK_RIGHT] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)]))
        {
            Cursor->SetX(Cursor->GetX()+1);
            Input->Delay();
        }




       /*
                case SDL_MOUSEBUTTONDOWN:
                    tmpx = Event.button.x / TileWidth;
                    tmpy = Event.button.y / TileHeight;
                    if((tmpx >= 0) && (tmpx < TILECOLUMNS) && (tmpy >=0) && (tmpy < TILEROWS))
                    {
                        prevx = tmpx;
                        prevy = tmpy;
                        Cursor->SetXY(tmpx,tmpy);
                    }
                    break;
                case SDL_MOUSEBUTTONUP:
                    tmpx = Event.button.x / TileWidth;
                    tmpy = Event.button.y / TileHeight;
                    if((tmpx >= 0) && (tmpy < TILECOLUMNS) && (tmpy >=0) && (tmpy < TILEROWS))
                    {
                        if(tmpx > prevx)
                        {
                            MovePiece(Cursor->GetX(),Cursor->GetY(),DM_RIGHT);
                            Cursor->SetXY(dm_srcx(),dm_srcy());
                        }
                        else
                            if(tmpx < prevx)
                            {
                                MovePiece(Cursor->GetX(),Cursor->GetY(),DM_LEFT);
                                Cursor->SetXY(dm_srcx(),dm_srcy());
                            }
                            else
                                if(tmpy > prevy)
                                {
                                    MovePiece(Cursor->GetX(),Cursor->GetY(),DM_DOWN);
                                    Cursor->SetXY(dm_srcx(),dm_srcy());
                                }
                                else
                                    if(tmpy < prevy)
                                    {
                                        MovePiece(Cursor->GetX(),Cursor->GetY(),DM_UP);
                                        Cursor->SetXY(dm_srcx(),dm_srcy());
                                    }
                    }
                    break;
*/


        if(dm_state() != DM_NORMAL)
        switch(dm_state())
        {
            case DM_FINISHED:
                if(StaticLevels)
                {
                    if(HighScores[lvl] == 0)
                        HighScores[lvl] = dm_movecount();
                    else
                        if(HighScores[lvl] > dm_movecount())
                            HighScores[lvl] = dm_movecount();
                    if(lvl+1<MAXSTATICLEVELS)
                        UnlockData[lvl+1] = '1';
                    SaveUnlockData();
                }
                GameState = GSWINMENU;
                break;
            case DM_OUT_OF_MOVES:
            case DM_HIT_DIE_PIECE:
            case DM_OUT_OF_FIELD:
            case DM_INFINITE_LOOP:
                GameState = GSDIEDMENU;
                break;
            default:
                break;
        }

        SDL_BlitSurface(Background,NULL,Buffer,NULL);
        DrawLevel(Buffer);
        Cursor->Draw(Buffer);
        if(StaticLevels)
            sprintf(Text,"Level: %d\nRecord: %d\nMoves: %d\nStones: %d",lvl+1,HighScores[lvl],dm_movecount(),dm_stones_left());
        else
            sprintf(Text,"Level: %d\nMoves: %d\nStones: %d",lvl+1,dm_movecount(),dm_stones_left());
        WriteText(Buffer,Font,Text,strlen(Text),242*SCREENPOSMULTIPLY,182*SCREENPOSMULTIPLY,0,Brown,false);
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif

        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);

        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);

    }
    delete Input;
}

void LevelEditorLevelSelect(void)
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    dm_u8 TmpPiece = NextPiece(DM_P_EMPTY);
    SDL_Surface *Tmp,*Tmp1;
    float size=0.01;
    int x,y,piecenr = 0;
    SDL_Rect Dst,Src;
    SDL_Event Event;
    int Selected = 0;
    char Text[256];
    SaveEnabled = false;
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(10);
    while (GameState == GSLEVELEDITORLEVELSELECT)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS]))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if( Input->KeyboardHeld[SDLK_ESCAPE] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)])
        {
            GameState = GSLEVELEDITORMENU;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);

        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Selected++;
            if (Selected > 3)
                Selected = 0;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            Selected--;
            if (Selected < 0)
                Selected = 3;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if((Input->Ready()) && ( Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)]))
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            switch (Selected)
            {
                case 0: // Next
                    lvl++;
                    if(lvl > UserLevelCount-1)
                        lvl = 0;
                    LoadLevel(UserLevelsFilenames[lvl]);
                    dm_init_level(UserLevel.field);
                    break;
                case 1: // previous
                    lvl--;
                    if(lvl < 0 )
                        lvl = UserLevelCount-1;
                    LoadLevel(UserLevelsFilenames[lvl]);
                    dm_init_level(UserLevel.field);
                    break;
                case 2: //edit
                    SaveEnabled = true;
                    Cursor->SetXY(8,8);
                    GameState=GSLEVELEDITOR;
                    break;
                case 3: //back
                    memset(UserLevel.field,0,256);
                    memset(UserLevel.name,0,256);
                    dm_init_level(UserLevel.field);
                    GameState = GSLEVELEDITORMENU;
                    break;

            }
            Input->Delay();
        }

        SDL_BlitSurface(BackgroundLevelEditor,NULL,Buffer,NULL);
        DrawLevel(Buffer);

            Tmp = zoomSurface(LevelTitle,1,1,0);
            sprintf(Text,"Level %d: %s",lvl+1,UserLevel.name);
            WriteText(Tmp,Font,Text,strlen(Text),0,8*SCREENPOSMULTIPLY,0,DarkBlue,true);
            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y = 15*SCREENPOSMULTIPLY;
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);

            Tmp = zoomSurface(Menu, 1, 1,0);

//            Dst.x = (Tmp->w/2) - (SelectLevelTitle->w/2);
//            Dst.y = 4;
//            Dst.w = SelectLevelTitle->w;
//            Dst.h = SelectLevelTitle->h;
//            SDL_BlitSurface(SelectLevelTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp,FontDark,4*SCREENPOSMULTIPLY,"Select Level");
            if(Selected ==0)
            {
//                Dst.x = (Tmp->w/2) - (Next1->w/2);
//                Dst.y = 24;
//                Dst.w = Next1->w;
//                Dst.h = Next1->h;
//                SDL_BlitSurface(Next1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,24*SCREENPOSMULTIPLY,"Next");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Next2->w/2);
//                Dst.y = 24;
//                Dst.w = Next2->w;
//                Dst.h = Next2->h;
//                SDL_BlitSurface(Next2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,24*SCREENPOSMULTIPLY,"Next");
            }

            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (Previous1->w/2);
//                Dst.y = 44;
//                Dst.w = Previous1->w;
//                Dst.h = Previous1->h;
//                SDL_BlitSurface(Previous1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,44*SCREENPOSMULTIPLY,"Previous");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Previous2->w/2);
//                Dst.y = 44;
//                Dst.w = Previous2->w;
//                Dst.h = Previous2->h;
//                SDL_BlitSurface(Previous2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,44*SCREENPOSMULTIPLY,"Previous");
            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (Play1->w/2);
//                Dst.y = 64;
//                Dst.w = Play1->w;
//                Dst.h = Play1->h;
//                SDL_BlitSurface(Play1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,64*SCREENPOSMULTIPLY,"Edit");
            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Play2->w/2);
//                Dst.y = 64;
//                Dst.w = Play2->w;
//                Dst.h = Play2->h;
//                SDL_BlitSurface(Play2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,64*SCREENPOSMULTIPLY,"Edit");
            }

            if(Selected == 3)
            {
//                Dst.x = (Tmp->w/2) - (Quit1->w/2);
//                Dst.y = 84;
//                Dst.w = Quit1->w;
//                Dst.h = Quit1->h;
//                SDL_BlitSurface(Quit1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,84*SCREENPOSMULTIPLY,"Back");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Quit2->w/2);
//                Dst.y = 84;
//                Dst.w = Quit2->w;
//                Dst.h = Quit2->h;
//                SDL_BlitSurface(Quit2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,84*SCREENPOSMULTIPLY,"Back");
            }
            if(StaticLevels)
                if(UnlockData[lvl]== '1')
                {
                    Dst.x = (Tmp->w/2) - (Unlocked->w/2);
                    Dst.y = 108*SCREENPOSMULTIPLY;
                    Dst.w = Unlocked->w;
                    Dst.h = Unlocked->h;
                    SDL_BlitSurface(Unlocked,NULL,Tmp,&Dst);
                }
                else
                {
                    Dst.x = (Tmp->w/2) - (Locked->w/2);
                    Dst.y = 108*SCREENPOSMULTIPLY;
                    Dst.w = Locked->w;
                    Dst.h = Locked->h;
                    SDL_BlitSurface(Locked,NULL,Tmp,&Dst);
                }

           if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y =(WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);

        TmpPiece = NextPiece(DM_P_EMPTY);
        for(y = 0;y<6;y++)
        {
            for(x=0;x<5;x++)
            {
                switch(L7(TmpPiece))
                {
                    case DM_P_RED:
                        if(SOLID(TmpPiece))
                            piecenr = 14;
                        else
                            piecenr = 30;
                        break;
                    case DM_P_BLUE:
                         if(SOLID(TmpPiece))
                            piecenr = 16;
                        else
                            piecenr = 32;
                        break;
                    case DM_P_GREEN:
                         if(SOLID(TmpPiece))
                            piecenr = 15;
                        else
                            piecenr = 31;
                        break;
                    case DM_P_YELLOW:
                        if(SOLID(TmpPiece))
                            piecenr = 13;
                        else
                            piecenr = 29;
                        break;

                    case DM_P_BLACK:
                        if(SOLID(TmpPiece))
                            piecenr = 12;
                        else
                            piecenr = 28;
                        break;

                    case DM_P_HORI:
                        if(SOLID(TmpPiece))
                            piecenr = 4;
                        else
                            piecenr = 20;
                        break;

                    case DM_P_VERT:
                        if(SOLID(TmpPiece))
                            piecenr = 3;
                        else
                            piecenr = 19;
                        break;

                    case DM_P_TELE1:
                        if(SOLID(TmpPiece))
                            piecenr = 5;
                        else
                            piecenr = 21;
                        break;

                    case DM_P_TELE2:
                        if(SOLID(TmpPiece))
                            piecenr = 6;
                        else
                            piecenr = 22;
                        break;

                    case DM_P_ROTCOL:
                        if(SOLID(TmpPiece))
                            piecenr = 7;
                        else
                            piecenr = 23;
                        break;

                    case DM_P_ROT_CW:
                        if(SOLID(TmpPiece))
                            piecenr = 8;
                        else
                            piecenr = 24;
                        break;

                    case DM_P_ROT_CCW:
                        if(SOLID(TmpPiece))
                            piecenr = 9;
                        else
                            piecenr = 25;
                        break;

                    case DM_P_GRAY:
                        if(SOLID(TmpPiece))
                            piecenr = 10;
                        else
                            piecenr = 26;
                        break;

                    case DM_P_DIE:
                        if(SOLID(TmpPiece))
                            piecenr = 11;
                        else
                            piecenr = 27;
                        break;

                    default:
                        piecenr=0;
                        break;
                }
                if(piecenr != 0)
                {
                    Src.x = 0;
                    Src.y = piecenr*TileHeight;
                    Src.w = TileWidth;
                    Src.h = TileHeight;
                    Dst.x = (x * TileWidth)+(243*SCREENPOSMULTIPLY);
                    Dst.y = (y * TileHeight)+(150*SCREENPOSMULTIPLY);
                    Dst.w = TileWidth;
                    Dst.h = TileHeight;
                    SDL_BlitSurface(LevelData,&Src,Buffer,&Dst);
                    //SDL_LockSurface(Buffer);
                    //SDL_UnlockSurface(Buffer);
//                    if(SelectedPiece == TmpPiece)
//                    {
//                        boxRGBA(Screen,(x * TileWidth)+243,(y * TileHeight)+150,(x * TileWidth)+243 + TileWidth-1,(y * TileHeight)+150+TileHeight-1,0,0,255,128);
//                    }
                }

                TmpPiece = NextPiece(TmpPiece);
                if(((y)*5)+(x) >= 27)
                    break;
            }
            if(((y)*5)+(x) >= 27)
                    break;
        }
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;

}

void LevelEditor(void)
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    dm_u8 SelectedPiece = NextPiece(DM_P_EMPTY),TmpPiece = NextPiece(DM_P_EMPTY);
    SDL_Event Event;
    int tmpx=0,tmpy=0,piecenr = 0,x,y;
    SDL_Rect Src,Dst;
    tmpx = Cursor->GetX();
    tmpy = Cursor->GetY();
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(5);
    while (GameState == GSLEVELEDITOR)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        Input->Update();

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if(Input->KeyboardHeld[SDLK_ESCAPE] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)])
        {
            GameState = GSLEVELEDITORMENU;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);

        }

        if( (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] ||
                                Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_b]  || Input->KeyboardHeld[SDLK_a]|| Input->KeyboardHeld[SDLK_q] ))
        {
            UserLevel.field[(Cursor->GetY() * TILEROWS) + Cursor->GetX()] = SelectedPiece;
            dm_init_level(UserLevel.field);
        }

        if( (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] ||
                                Input->KeyboardHeld[SDLK_BACKSPACE] || Input->KeyboardHeld[SDLK_x]  || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_DELETE] ))
        {
            UserLevel.field[(Cursor->GetY() * TILEROWS) + Cursor->GetX()] = DM_P_EMPTY;
            dm_init_level(UserLevel.field);
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_L)] || Input->KeyboardHeld[SDLK_PAGEDOWN] || Input->KeyboardHeld[SDLK_l]))
        {
            SelectedPiece = PreviousPiece(SelectedPiece);
            Input->Delay();
        }

        if((Input->Ready()) && ( Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_PAGEUP] || Input->KeyboardHeld[SDLK_r]))
        {
            SelectedPiece = NextPiece(SelectedPiece);
            Input->Delay();
        }

        if((Input->Ready()) && ((Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]) &&
                               (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT])))
        {
            Cursor->SetY(Cursor->GetY()-1);
            Cursor->SetX(Cursor->GetX()-1);
            Input->Delay();
        }

        if((Input->Ready()) && ((Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]) &&
                               (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT])))
        {
            Cursor->SetY(Cursor->GetY()+1);
            Cursor->SetX(Cursor->GetX()-1);
            Input->Delay();
        }

        if((Input->Ready()) && ((Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]) &&
                               (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP])) )
        {
            Cursor->SetY(Cursor->GetY()-1);
            Cursor->SetX(Cursor->GetX()+1);
            Input->Delay();
        }

        if((Input->Ready()) && ((Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]) &&
                               (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN])) )
        {
            Cursor->SetY(Cursor->GetY()+1);
            Cursor->SetX(Cursor->GetX()+1);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
        {
            Cursor->SetY(Cursor->GetY()-1);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
        {
            Cursor->SetY(Cursor->GetY()+1);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
        {
            Cursor->SetX(Cursor->GetX()-1);
            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
        {
            Cursor->SetX(Cursor->GetX()+1);
            Input->Delay();
        }

        SDL_BlitSurface(BackgroundLevelEditor,NULL,Buffer,NULL);
        DrawLevel(Buffer);
        Cursor->Draw(Buffer);
        TmpPiece = NextPiece(DM_P_EMPTY);
        for(y = 0;y<6;y++)
        {
            for(x=0;x<5;x++)
            {
                switch(L7(TmpPiece))
                {
                    case DM_P_RED:
                        if(SOLID(TmpPiece))
                            piecenr = 14;
                        else
                            piecenr = 30;
                        break;
                    case DM_P_BLUE:
                         if(SOLID(TmpPiece))
                            piecenr = 16;
                        else
                            piecenr = 32;
                        break;
                    case DM_P_GREEN:
                         if(SOLID(TmpPiece))
                            piecenr = 15;
                        else
                            piecenr = 31;
                        break;
                    case DM_P_YELLOW:
                        if(SOLID(TmpPiece))
                            piecenr = 13;
                        else
                            piecenr = 29;
                        break;

                    case DM_P_BLACK:
                        if(SOLID(TmpPiece))
                            piecenr = 12;
                        else
                            piecenr = 28;
                        break;

                    case DM_P_HORI:
                        if(SOLID(TmpPiece))
                            piecenr = 4;
                        else
                            piecenr = 20;
                        break;

                    case DM_P_VERT:
                        if(SOLID(TmpPiece))
                            piecenr = 3;
                        else
                            piecenr = 19;
                        break;

                    case DM_P_TELE1:
                        if(SOLID(TmpPiece))
                            piecenr = 5;
                        else
                            piecenr = 21;
                        break;

                    case DM_P_TELE2:
                        if(SOLID(TmpPiece))
                            piecenr = 6;
                        else
                            piecenr = 22;
                        break;

                    case DM_P_ROTCOL:
                        if(SOLID(TmpPiece))
                            piecenr = 7;
                        else
                            piecenr = 23;
                        break;

                    case DM_P_ROT_CW:
                        if(SOLID(TmpPiece))
                            piecenr = 8;
                        else
                            piecenr = 24;
                        break;

                    case DM_P_ROT_CCW:
                        if(SOLID(TmpPiece))
                            piecenr = 9;
                        else
                            piecenr = 25;
                        break;

                    case DM_P_GRAY:
                        if(SOLID(TmpPiece))
                            piecenr = 10;
                        else
                            piecenr = 26;
                        break;

                    case DM_P_DIE:
                        if(SOLID(TmpPiece))
                            piecenr = 11;
                        else
                            piecenr = 27;
                        break;

                    default:
                        piecenr=0;
                        break;
                }
                if(piecenr != 0)
                {
                    Src.x = 0;
                    Src.y = piecenr*TileHeight;
                    Src.w = TileWidth;
                    Src.h = TileHeight;
                    Dst.x = (x * TileWidth)+(243*SCREENPOSMULTIPLY);
                    Dst.y = (y * TileHeight)+(150*SCREENPOSMULTIPLY);
                    Dst.w = TileWidth;
                    Dst.h = TileHeight;
                    SDL_BlitSurface(LevelData,&Src,Buffer,&Dst);
                    //SDL_LockSurface(Buffer);
                    //SDL_UnlockSurface(Buffer);
                    if(SelectedPiece == TmpPiece)
                    {
                        boxRGBA(Buffer,(x * TileWidth)+(243*SCREENPOSMULTIPLY),(y * TileHeight)+(150*SCREENPOSMULTIPLY),(x * TileWidth)+(243*SCREENPOSMULTIPLY) + TileWidth-1,(y * TileHeight)+(150*SCREENPOSMULTIPLY)+TileHeight-1,0,0,255,128);
                    }
                }

                TmpPiece = NextPiece(TmpPiece);
                if(((y)*5)+(x) >= 27)
                    break;
            }
            if(((y)*5)+(x) >= 27)
                    break;
        }
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}

void UpdateUserLevelFilenames(char *filename)
{
    int teller;
    bool found = false;
    for (teller =0;teller< UserLevelCount;teller++)
    {
        if(strcmp(filename, UserLevelsFilenames[teller])==0)
        {
            found = true;
            break;
        }
    }
    if(!found)
    {
        UserLevelCount++;
        strcpy(UserLevelsFilenames[UserLevelCount-1],filename);
        lvl = UserLevelCount;
    }

}

int CheckForLevelErrors()
{
    int x,y,NumTeleport1=0,NumTeleport2=0,NumColors=0,NumObjects=0,tmp;
    for(y=0;y<TILEROWS;y++)
    {
        for(x=0;x<TILECOLUMNS;x++)
        {
            tmp = dm_field()[(y*TILEROWS)+x];
            switch(L7(tmp))
            {
                case DM_P_DIE:
                case DM_P_GRAY:
                case DM_P_ROT_CCW:
                case DM_P_ROT_CW:
                case DM_P_ROTCOL:
                case DM_P_VERT:
                case DM_P_HORI:
                    NumObjects++;
                    break;

                case DM_P_BLACK:
                case DM_P_YELLOW:
                case DM_P_GREEN:
                case DM_P_BLUE:
                case DM_P_RED:
                    if(SOLID(tmp))
                        NumObjects++;
                    else
                    {
                        NumObjects++;
                        NumColors++;
                    }
                    break;
                case DM_P_TELE1:
                    NumTeleport1++;
                    NumObjects++;
                    break;

                case DM_P_TELE2:
                    NumTeleport2++;
                    NumObjects++;
                    break;
                default :
                    break;
            }
        }
    }

    if(NumTeleport1 > 0)
        if(NumTeleport1 != 2)
            return 0; //Teleport 1 invalid number
    if(NumTeleport2 > 0)
        if(NumTeleport2 != 2)
            return 1; //teleport 2 invalid number
    if(NumObjects == 0)
        return 2; // no pieces in the field
    if(NumColors < 2)
        return 3; // there must be at least 2 colored pieces
    return -1; //all good
}

void LevelEditorMenu(void)
{
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    char Text[256],Text1[512];
    dm_u8 TmpPiece = NextPiece(DM_P_EMPTY);
    SDL_Surface *Tmp,*Tmp1;
    int x,y,piecenr=0,LevelError = -1;
    float size=0.01;
    SDL_Event Event;
    SDL_Rect Dst,Src;
    int Selected = 0;
    while (SDL_PollEvent(&Event));
    CInput *Input = new CInput(10);
    while (GameState == GSLEVELEDITORMENU)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        if(size >= 1)
            Input->Update();

        if ((Input->Ready()) && ( Input->KeyboardHeld[SDLK_KP_MINUS] ))
        {
            DecVolume();
            Input->Delay();
        }

        if ((Input->Ready()) && (Input->KeyboardHeld[SDLK_KP_PLUS] ))
        {
            IncVolume();
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

        if(Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)] || Input->KeyboardHeld[SDLK_ESCAPE])
        {
            SaveEnabled = true;
            GameState = GSLEVELEDITOR;
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
        }

        if((Input->Ready()) && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Selected++;
            if (Selected > 4)
                Selected = 0;
                if(!SaveEnabled)
                    if(Selected == 2)
                        Selected = 4;
            if(UserLevelCount == 0)
            {
                if(Selected == 1)
                    if(SaveEnabled)
                        Selected = 2;
                    else
                        Selected = 4;

            }
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if((Input->Ready()) && ( Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            Selected--;
            if (Selected < 0)
                Selected = 4;
                if(!SaveEnabled)
                    if(Selected == 3)
                        Selected = 1;

            if(UserLevelCount == 0)
            {
                if(Selected == 1)
                    Selected = 0;

            }
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();

        }

        if(Input->KeyboardHeld[SDLK_SPACE] ||
           Input->KeyboardHeld[SDLK_RETURN] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_a]|| Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] )
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            switch (Selected)
            {
                case 0: // new
                    memset(UserLevel.name,0,256);
                    memset(UserLevel.hs,0,256);
                    memset(UserLevel.field,0,256);
                    dm_init_level(UserLevel.field);
                    SaveEnabled = true;
                    GameState = GSLEVELEDITOR;
                    Cursor->SetXY(8,8);
                    break;
                case 1: //level select
                    lvl = 0;
                    LoadLevel(UserLevelsFilenames[lvl]);
                    dm_init_level(UserLevel.field);
                    GameState = GSLEVELEDITORLEVELSELECT;
                    break;
                case 2 :
                    LevelError = CheckForLevelErrors() ;
                    if(LevelError >= 0)
                    {
                        switch(LevelError)
                        {
                            case 0:
                                PrintMsg("When using the teleporters there\nshould be two teleporter pieces\nof each kind!\n\nPlease fix this error before saving!","Level Error");
                            break;
                            case 1:
                                PrintMsg("When using the teleporters there\nshould be two teleporter pieces\nof each kind!\n\nPlease fix this error before saving!","Level Error");
                            break;
                            case 2:
                                PrintMsg("There are no pieces in this level!\n\nPlease fix this error before saving!","Level Error");
                            break;
                            case 3:
                                PrintMsg("There should be at least 2 colored\npieces in a level !\n\nPlease fix this error before saving!","Level Error");
                            break;

                        }
                    }
                    else
                    {
                        memset(Text,0,256);
                        sprintf(Text,"%s",UserLevel.name);
                        if(GetName(Text))
                        {
                            memset(UserLevel.name,0,256);
                            sprintf(UserLevel.name , "%s",Text);
                            sprintf(Text1,"./userlevels/%s.lev",Text);
                            SaveLevel(Text1);
                            UpdateUserLevelFilenames(Text1);
                        }
                    }
                    Input->Reset();
                    break;
                case 3:
                    GameState = GSLEVELEDITOR;
                    break;
                case 4: //Quit
                    GameState = GSTITLESCREEN;
                    break;
            }
        }

        SDL_BlitSurface(BackgroundLevelEditor,NULL,Buffer,NULL);
        DrawLevel(Buffer);


            Tmp = zoomSurface(Menu, 1, 1,0);
//            Dst.x = (Tmp->w/2) - (GameMenuTitle->w/2);
//            Dst.y = 4;
//            Dst.w = GameMenuTitle->w;
//            Dst.h = GameMenuTitle->h;
//            SDL_BlitSurface(GameMenuTitle,NULL,Tmp,&Dst);
            SFont_WriteCenter(Tmp,FontDark,4*SCREENPOSMULTIPLY,"Level Editor");
            if(Selected == 0)
            {
//                Dst.x = (Tmp->w/2) - (ResumePlay1->w/2);
//                Dst.y = 27;
//                Dst.w = ResumePlay1->w;
//                Dst.h = ResumePlay1->h;
//                SDL_BlitSurface(ResumePlay1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,27*SCREENPOSMULTIPLY,"New");
            }
            else
            {
//                Dst.x = (Tmp->w/2) - (ResumePlay2->w/2);
//                Dst.y = 27;
//                Dst.w = ResumePlay2->w;
//                Dst.h = ResumePlay2->h;
//                SDL_BlitSurface(ResumePlay2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,27*SCREENPOSMULTIPLY,"New");
            }

            if(Selected == 1)
            {
//                Dst.x = (Tmp->w/2) - (Retry1->w/2);
//                Dst.y = 50;
//                Dst.w = Retry1->w;
//                Dst.h = Retry1->h;
//                SDL_BlitSurface(Retry1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,46*SCREENPOSMULTIPLY,"Select Level");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Retry2->w/2);
//                Dst.y = 50;
//                Dst.w = Retry2->w;
//                Dst.h = Retry2->h;
//                SDL_BlitSurface(Retry2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,46*SCREENPOSMULTIPLY,"Select Level");
            }

            if(Selected == 2)
            {
//                Dst.x = (Tmp->w/2) - (Retry1->w/2);
//                Dst.y = 50;
//                Dst.w = Retry1->w;
//                Dst.h = Retry1->h;
//                SDL_BlitSurface(Retry1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,65*SCREENPOSMULTIPLY,"Save");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (Retry2->w/2);
//                Dst.y = 50;
//                Dst.w = Retry2->w;
//                Dst.h = Retry2->h;
//                SDL_BlitSurface(Retry2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,65*SCREENPOSMULTIPLY,"Save");
            }
            if(Selected == 3)
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel1->w/2);
//                Dst.y = 95;
//                Dst.w = SelectLevel1->w;
//                Dst.h = SelectLevel1->h;
//                SDL_BlitSurface(SelectLevel1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,83*SCREENPOSMULTIPLY,"Back");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel2->w/2);
//                Dst.y = 95;
//                Dst.w = SelectLevel2->w;
//                Dst.h = SelectLevel2->h;
//                SDL_BlitSurface(SelectLevel2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,83*SCREENPOSMULTIPLY,"Back");
            }
            if(Selected == 4)
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel1->w/2);
//                Dst.y = 95;
//                Dst.w = SelectLevel1->w;
//                Dst.h = SelectLevel1->h;
//                SDL_BlitSurface(SelectLevel1,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontDark,101*SCREENPOSMULTIPLY,"Quit");

            }
            else
            {
//                Dst.x = (Tmp->w/2) - (SelectLevel2->w/2);
//                Dst.y = 95;
//                Dst.w = SelectLevel2->w;
//                Dst.h = SelectLevel2->h;
//                SDL_BlitSurface(SelectLevel2,NULL,Tmp,&Dst);
                SFont_WriteCenter(Tmp,FontLight,101*SCREENPOSMULTIPLY,"Quit");
            }


            if(size < 1)
            {
                Tmp1 = zoomSurface(Tmp,size,size,0);
            }
            else
            {
                Tmp1 = zoomSurface(Tmp,1,1,0);
            }
            Dst.x = 120*SCREENPOSMULTIPLY-(Tmp1->w / 2);
            Dst.y =(WINDOW_HEIGHT/2) -(Tmp1->h / 2);
            Dst.w = Tmp1->w;
            Dst.h = Tmp1->h;
            SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
            if(Tmp1)
                SDL_FreeSurface(Tmp1);
            if(Tmp)
                SDL_FreeSurface(Tmp);
        TmpPiece = NextPiece(DM_P_EMPTY);
        for(y = 0;y<6;y++)
        {
            for(x=0;x<5;x++)
            {
                switch(L7(TmpPiece))
                {
                    case DM_P_RED:
                        if(SOLID(TmpPiece))
                            piecenr = 14;
                        else
                            piecenr = 30;
                        break;
                    case DM_P_BLUE:
                         if(SOLID(TmpPiece))
                            piecenr = 16;
                        else
                            piecenr = 32;
                        break;
                    case DM_P_GREEN:
                         if(SOLID(TmpPiece))
                            piecenr = 15;
                        else
                            piecenr = 31;
                        break;
                    case DM_P_YELLOW:
                        if(SOLID(TmpPiece))
                            piecenr = 13;
                        else
                            piecenr = 29;
                        break;

                    case DM_P_BLACK:
                        if(SOLID(TmpPiece))
                            piecenr = 12;
                        else
                            piecenr = 28;
                        break;

                    case DM_P_HORI:
                        if(SOLID(TmpPiece))
                            piecenr = 4;
                        else
                            piecenr = 20;
                        break;

                    case DM_P_VERT:
                        if(SOLID(TmpPiece))
                            piecenr = 3;
                        else
                            piecenr = 19;
                        break;

                    case DM_P_TELE1:
                        if(SOLID(TmpPiece))
                            piecenr = 5;
                        else
                            piecenr = 21;
                        break;

                    case DM_P_TELE2:
                        if(SOLID(TmpPiece))
                            piecenr = 6;
                        else
                            piecenr = 22;
                        break;

                    case DM_P_ROTCOL:
                        if(SOLID(TmpPiece))
                            piecenr = 7;
                        else
                            piecenr = 23;
                        break;

                    case DM_P_ROT_CW:
                        if(SOLID(TmpPiece))
                            piecenr = 8;
                        else
                            piecenr = 24;
                        break;

                    case DM_P_ROT_CCW:
                        if(SOLID(TmpPiece))
                            piecenr = 9;
                        else
                            piecenr = 25;
                        break;

                    case DM_P_GRAY:
                        if(SOLID(TmpPiece))
                            piecenr = 10;
                        else
                            piecenr = 26;
                        break;

                    case DM_P_DIE:
                        if(SOLID(TmpPiece))
                            piecenr = 11;
                        else
                            piecenr = 27;
                        break;

                    default:
                        piecenr=0;
                        break;
                }
                if(piecenr != 0)
                {
                    Src.x = 0;
                    Src.y = piecenr*TileHeight;
                    Src.w = TileWidth;
                    Src.h = TileHeight;
                    Dst.x = (x * TileWidth)+(243*SCREENPOSMULTIPLY);
                    Dst.y = (y * TileHeight)+(150*SCREENPOSMULTIPLY);
                    Dst.w = TileWidth;
                    Dst.h = TileHeight;
                    SDL_BlitSurface(LevelData,&Src,Buffer,&Dst);
//                    if(SelectedPiece == TmpPiece)
//                    {
//                        boxRGBA(Screen,(x * TileWidth)+243,(y * TileHeight)+150,(x * TileWidth)+243 + TileWidth-1,(y * TileHeight)+150+TileHeight-1,0,0,255,128);
//                    }
                    //SDL_LockSurface(Buffer);
                    //SDL_UnlockSurface(Buffer);
                }

                TmpPiece = NextPiece(TmpPiece);
                if(((y)*5)+(x) >= 27)
                    break;
            }
            if(((y)*5)+(x) >= 27)
                    break;
        }
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);
    }
    delete Input;
}

void Intro()
{
    SDL_Rect Rect;
    Uint8 alpha = 0;
    Uint32 Time1=0;
    int IntroScreenNr = 1;
    const int   MaxAlpha = 150,AlphaIncrease = 20;
    SDL_Surface *Tmp;
#if defined GP2X || defined CAANOO
    Tmp = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
#else
    Tmp = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,32,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
#endif
    CInput *Input = new CInput(10);
    Time1 = SDL_GetTicks();
    while (GameState == GSINTRO)
    {
        Input->Update();

        if(Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] || Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_A)] ||
           Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_X)] ||  Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_SELECT)] ||  Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_START)] )
        {
            GameState=GSTITLESCREEN;
        }

        if( Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r])
        {
            if(Input->Ready() && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
            {
                if(StartScreenX - 2 >= 0)
                    StartScreenX -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
            {
                if(StartScreenX + 2 + Buffer->w <= Screen->w)
                    StartScreenX +=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
            {
                if(StartScreenY - 2 >=0)
                    StartScreenY -=2;
                Input->Delay();
            }

            if(Input->Ready() && ( Input->JoystickHeld[0][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
            {
                if(StartScreenY + 2 + Buffer->h <= Screen->h)
                    StartScreenY +=2;
                Input->Delay();
            }
        }

        switch(IntroScreenNr)
        {
            case 1 :
                SDL_BlitSurface(IMGIntro1,NULL,Tmp,NULL);
                break;
            case 2 :
                SDL_BlitSurface(IMGIntro2,NULL,Tmp,NULL);
                break;
            case 3 :
                SDL_BlitSurface(IMGIntro3,NULL,Tmp,NULL);
                break;
            case 4 :
                SDL_BlitSurface(IMGIntro4,NULL,Tmp,NULL);
                break;
        }

        if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaIncrease;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }

        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;

        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }

        if(Time1 + 3700 < SDL_GetTicks())
        {
            alpha = 0;
            IntroScreenNr++;
            if(IntroScreenNr > 4)
                GameState = GSTITLESCREEN;
            Time1 = SDL_GetTicks();
        }

    }
    SDL_FreeSurface(Tmp);
    delete Input;
}

void SetupUsbJoystickButtons()
{

    int Teller, Selection = 0;
	SDL_Rect Rect;
    SDL_Surface *Tmp,*Tmp1;
    float size=0.01;
    SDL_Rect Dst;
    CInput *Input = new CInput(10);
	bool done = false;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	while (GameState == GSSETUPUSBJOYSTICKBUTTONS)
	{
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

		Input->Update();

		if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQUIT;

		if(Input->KeyboardHeld[SDLK_ESCAPE] ||  Input->JoystickHeld[0][6])
            GameState = GSTITLESCREEN;

        if(Input->Ready() && ( Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[0][JOYSTICK_UP]))
        {
            Selection--;
            if( Selection < 0)
                Selection = NROFBUTTONS-1;
            if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if (Input->JoystickHeld[0][8] || Input->KeyboardHeld[SDLK_RETURN])
            JoystickSetup->ResetToDefaults();

        if(Input->Ready() && (Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[0][JOYSTICK_DOWN]))
        {
            Selection++;
            if( Selection >=NROFBUTTONS)
                Selection = 0;
            if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if(Input->Ready() && (Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_SPACE] || Input->JoystickHeld[0][0]))
        {
            if (size >= 1)
            {
                if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                Input->Reset();
                while (!done)
                {
                    Input->Update();
                    Tmp = zoomSurface(Menu, 1.4, 1.25,0);
            //            Dst.x = (Tmp->w/2)  - (MainMenuTitle->w/2);
            //            Dst.y = 4;
            //            Dst.w = MainMenuTitle->w;
            //            Dst.h = MainMenuTitle->h;
            //            //SDL_BlitSurface(MainMenuTitle,NULL,Tmp,&Dst);
                    SFont_WriteCenter(Tmp, FontDark, 7*SCREENPOSMULTIPLY,"Joystick Button Setup");
                    JoystickSetup->DrawCurrentSetup(Tmp,SmallFont,15*SCREENPOSMULTIPLY,30*SCREENPOSMULTIPLY,145*SCREENPOSMULTIPLY,10*SCREENPOSMULTIPLY,Selection,DarkBlue,Color);
                    Tmp1 = zoomSurface(Tmp,1,1,0);
                    Dst.x = (WINDOW_WIDTH/2) -(Tmp1->w / 2);
                    Dst.y = 140*SCREENPOSMULTIPLY -(Tmp1->h / 2);
                    Dst.w = Tmp1->w;
                    Dst.h = Tmp1->h;
                    SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
                    if(Tmp1)
                        SDL_FreeSurface(Tmp1);
                    if(Tmp)
                        SDL_FreeSurface(Tmp);

#ifdef SHOWFPS
                    sprintf(TextFps,"Fps:%d",fps);
                    WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
                    SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
                    //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
                    SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
                    SDL_Flip(Screen);
                    framecounter++;
                    if(Time + 1000 < SDL_GetTicks())
                    {
                        fps = framecounter;
                        framecounter = 0;
                        Time = SDL_GetTicks();
                    }

                    for (Teller = 0;Teller<MAXJOYSTICKBUTTONS;Teller++)
                        if(Input->JoystickHeld[0][Teller])
                        {
                            done = true;
                            JoystickSetup->SetButtonValue(Selection,Teller);
                            break;
                        }

                    if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
                    {
                        GameState = GSQUIT;
                        done = true;
                    }

                    if(Input->Ready() && (Input->JoystickHeld[0][0] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_SPACE]))
                        done = true;

                    if(Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[0][6])
                        done= true;
                    SDL_framerateDelay(&Fpsman);
                }
                Input->Reset();
                Input->Delay();
                done= false;
            }
        }
                SDL_BlitSurface(TitleScreen,NULL,Buffer,NULL);


        Tmp = zoomSurface(Menu, 1.4, 1.25,0);
//            Dst.x = (Tmp->w/2)  - (MainMenuTitle->w/2);
//            Dst.y = 4;
//            Dst.w = MainMenuTitle->w;
//            Dst.h = MainMenuTitle->h;
//            //SDL_BlitSurface(MainMenuTitle,NULL,Tmp,&Dst);
        SFont_WriteCenter(Tmp, FontDark, 7*SCREENPOSMULTIPLY,"Joystick Button Setup");
        JoystickSetup->DrawCurrentSetup(Tmp,SmallFont,15*SCREENPOSMULTIPLY,30*SCREENPOSMULTIPLY,145*SCREENPOSMULTIPLY,10*SCREENPOSMULTIPLY,Selection,DarkBlue,DarkBlue);
        if(size < 1)
        {
            Tmp1 = zoomSurface(Tmp,size,size,0);
        }
        else
        {
            Tmp1 = zoomSurface(Tmp,1,1,0);
        }
        Dst.x = (WINDOW_WIDTH/2) -(Tmp1->w / 2);
        Dst.y = 140*SCREENPOSMULTIPLY -(Tmp1->h / 2);
        Dst.w = Tmp1->w;
        Dst.h = Tmp1->h;
        SDL_BlitSurface(Tmp1,NULL,Buffer,&Dst);
        if(Tmp1)
            SDL_FreeSurface(Tmp1);
        if(Tmp)
            SDL_FreeSurface(Tmp);

#ifdef SHOWFPS
        sprintf(TextFps,"Fps:%d",fps);
        WriteText(Buffer,Font,TextFps,strlen(TextFps),0,0,0,Color,false);
#endif
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        //SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
        framecounter++;
        if(Time + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time = SDL_GetTicks();
        }
        if(size < 1)
            size+=SizeIncrease;
        //SDL_Delay(WaitForFrame());
        SDL_framerateDelay(&Fpsman);

	}
	delete Input;
}


void LoadGraphics(void) {
    SDL_Surface *Tmp,*Tmp2;
    Tmp = IMG_Load("./graphics/pieces.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        LevelData = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        LevelData = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(LevelData,SDL_SRCCOLORKEY,SDL_MapRGB(LevelData->format,255,0,255));


    Tmp = IMG_Load("./graphics/background.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        Background = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        Background = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/backgroundleveleditor.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        BackgroundLevelEditor = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        BackgroundLevelEditor = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }



    Tmp = IMG_Load("./graphics/menu.png");
    SDL_SetColorKey(Tmp ,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
    SDL_SetAlpha(Tmp ,SDL_SRCALPHA,230);

    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        Menu = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        Menu = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/leveltitle.png");
    SDL_SetColorKey(Tmp,SDL_SRCCOLORKEY,SDL_MapRGB(Tmp->format,255,0,255));
    SDL_SetAlpha(Tmp,SDL_SRCALPHA,230);
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        LevelTitle = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        LevelTitle = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }



//    Tmp = IMG_Load("./graphics/selectleveltitle.png");
//    SelectLevelTitle = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/gamemenutitle.png");
//    GameMenuTitle = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/youdiedtitle.png");
//    YouDiedTitle = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/quit1.png");
//    Quit1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/quit2.png");
//    Quit2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/retry1.png");
//    Retry1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/retry2.png");
//    Retry2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/resumeplay1.png");
//    ResumePlay1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/resumeplay2.png");
//    ResumePlay2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/lastunlocked1.png");
//    LastUnlocked1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/lastunlocked2.png");
//    LastUnlocked2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/next1.png");
//    Next1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/next2.png");
//    Next2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/previous1.png");
//    Previous1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/previous2.png");
//    Previous2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);

    Tmp = IMG_Load("./graphics/unlocked.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        Unlocked = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        Unlocked = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/locked.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        Locked = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        Locked = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }



//    Tmp = IMG_Load("./graphics/selectlevel1.png");
//    SelectLevel1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//
//    Tmp = IMG_Load("./graphics/selectlevel2.png");
//    SelectLevel2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/play1.png");
//    Play1= SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/play2.png");
//    Play2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/youwontitle.png");
//    YouWonTitle = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);

    Tmp = IMG_Load("./graphics/pieceoutsideplayfield.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        PieceOutsidePlayfield = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        PieceOutsidePlayfield = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/infiniteloop.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        InfiniteLoop = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        InfiniteLoop = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/deathpiece.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        DeathPiece = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        DeathPiece = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/outofmoves.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        OutOfMoves = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        OutOfMoves = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/levelcompleted.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        LevelCompleted =  zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        LevelCompleted = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/alllevelscompleted.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
        AllLevelsCompleted = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        AllLevelsCompleted = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }

//    Tmp = IMG_Load("./graphics/mainmenutitle.png");
//    MainMenuTitle = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/userlevels1.png");
//    UserLevels1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/userlevels2.png");
//    UserLevels2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/staticlevels1.png");
//    StaticLevels1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/staticlevels2.png");
//    StaticLevels2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/leveleditor1.png");
//    LevelEditor1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/leveleditor2.png");
//    LevelEditor2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/back1.png");
//    Back1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/back2.png");
//    Back2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/new1.png");
//    New1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/new2.png");
//    New2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/edit1.png");
//    Edit1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/edit2.png");
//    Edit2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/save1.png");
//    Save1 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/save2.png");
//    Save2 = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);
//
//    Tmp = IMG_Load("./graphics/leveleditortitle.png");
//    LevelEditorTitle = SDL_DisplayFormatAlpha(Tmp);
//    SDL_FreeSurface(Tmp);

    Tmp = IMG_Load("./graphics/darkfont14pt.png");
    if(SCREENPOSMULTIPLY > 1)
    {
     /*  Tmp2 = zoomSurface(Tmp,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,0);
       SDL_FreeSurface(Tmp);
       SFontDark = SDL_DisplayFormatAlpha(Tmp2);
       SDL_FreeSurface(Tmp2);*/
        Tmp2 = ScaleSurface(Tmp,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp);
        for(int y=1;y<SCREENPOSMULTIPLY;y++)
            memcpy((pixel *)Tmp2->pixels + (y)*Tmp2->pitch/SCREEN_BYTE,(pixel *)Tmp2->pixels+(Tmp2->h-1)*Tmp2->pitch/SCREEN_BYTE,Tmp2->w*SCREEN_BYTE);
        SFontDark = SDL_DisplayFormatAlpha(Tmp2);
        SDL_FreeSurface(Tmp2);
       //SDL_FreeSurface(Tmp2);
    }
    else
    {
        SFontDark = SDL_DisplayFormatAlpha(Tmp);
        SDL_FreeSurface(Tmp);
    }




    Tmp = IMG_Load("./graphics/lightfont14pt.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = ScaleSurface(Tmp,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp);
        for(int y=1;y<SCREENPOSMULTIPLY;y++)
            memcpy((pixel *)Tmp2->pixels + (y)*Tmp2->pitch/SCREEN_BYTE,(pixel *)Tmp2->pixels+(Tmp2->h-1)*Tmp2->pitch/SCREEN_BYTE,Tmp2->w*SCREEN_BYTE);
        SFontLight = SDL_DisplayFormatAlpha(Tmp2);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        SFontLight = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/titlescreen.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        TitleScreen = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        TitleScreen = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }


    Tmp = IMG_Load("./graphics/intro1.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGIntro1 = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGIntro1 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/intro2.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGIntro2 = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGIntro2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/intro3.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGIntro3 = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGIntro3 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    Tmp = IMG_Load("./graphics/intro4.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGIntro4 = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGIntro4 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    FontDark = SFont_InitFont(SFontDark);
    FontLight = SFont_InitFont(SFontLight);


}

void UnLoadGraphics(void){

    SFont_FreeFont(FontDark);

    SFont_FreeFont(FontLight);


   /* if(SFontLight)
        SDL_FreeSurface(SFontLight);

    if(SFontDark)
        SDL_FreeSurface(SFontDark);
*/

    if(IMGIntro1)
        SDL_FreeSurface(IMGIntro1);
    if(IMGIntro2)
        SDL_FreeSurface(IMGIntro2);
    if(IMGIntro3)
        SDL_FreeSurface(IMGIntro3);
    if(IMGIntro4)
        SDL_FreeSurface(IMGIntro4);

    if(LevelData)
        SDL_FreeSurface(LevelData);
    if(Background)
        SDL_FreeSurface(Background);

    if(BackgroundLevelEditor)
        SDL_FreeSurface(BackgroundLevelEditor);

    if(Menu)
        SDL_FreeSurface(Menu);

//    if(SelectLevelTitle)
//        SDL_FreeSurface(SelectLevelTitle);
//
//    if(GameMenuTitle)
//        SDL_FreeSurface(GameMenuTitle);
//
//    if(YouDiedTitle)
//        SDL_FreeSurface(YouDiedTitle);
//
//    if(Quit1)
//        SDL_FreeSurface(Quit1);
//
//    if(Quit2)
//        SDL_FreeSurface(Quit2);
//
//    if(Retry1)
//        SDL_FreeSurface(Retry1);
//
//    if(Retry2)
//        SDL_FreeSurface(Retry2);
//
//    if(ResumePlay1)
//        SDL_FreeSurface(ResumePlay1);
//
//    if(ResumePlay2)
//        SDL_FreeSurface(ResumePlay2);
//
//    if(LastUnlocked1)
//        SDL_FreeSurface(LastUnlocked1);
//
//    if(LastUnlocked2)
//        SDL_FreeSurface(LastUnlocked2);
//
//    if(Next1)
//        SDL_FreeSurface(Next1);
//
//    if(Next2)
//        SDL_FreeSurface(Next2);
//
//    if(Previous1)
//        SDL_FreeSurface(Previous1);
//
//    if(Previous2)
//        SDL_FreeSurface(Previous2);

    if(Unlocked)
        SDL_FreeSurface(Unlocked);

    if(Locked)
        SDL_FreeSurface(Locked);

//    if(SelectLevel1)
//        SDL_FreeSurface(SelectLevel1);
//
//    if(SelectLevel2)
//        SDL_FreeSurface(SelectLevel2);
//
//    if(Play1)
//        SDL_FreeSurface(Play1);
//
//    if(Play2)
//        SDL_FreeSurface(Play2);
//
//     if(YouWonTitle)
//        SDL_FreeSurface(YouWonTitle);

     if(PieceOutsidePlayfield)
        SDL_FreeSurface(PieceOutsidePlayfield);

     if(InfiniteLoop)
        SDL_FreeSurface(InfiniteLoop);

     if(DeathPiece)
        SDL_FreeSurface(DeathPiece);

     if(OutOfMoves)
        SDL_FreeSurface(OutOfMoves);

     if(LevelCompleted)
        SDL_FreeSurface(LevelCompleted);

    if(AllLevelsCompleted)
        SDL_FreeSurface(AllLevelsCompleted);

//    if(MainMenuTitle)
//        SDL_FreeSurface(MainMenuTitle);
//
//    if(UserLevels1)
//        SDL_FreeSurface(UserLevels1);
//
//    if(UserLevels2)
//        SDL_FreeSurface(UserLevels2);
//
//    if(StaticLevels1)
//        SDL_FreeSurface(StaticLevels1);
//
//    if(StaticLevels2)
//        SDL_FreeSurface(StaticLevels2);
//
//    if(LevelEditor1)
//        SDL_FreeSurface(LevelEditor1);
//
//    if(LevelEditor2)
//        SDL_FreeSurface(LevelEditor2);

    if(LevelTitle)
        SDL_FreeSurface(LevelTitle);

    if(TitleScreen)
        SDL_FreeSurface(TitleScreen);

//    if(Back1)
//        SDL_FreeSurface(Back1);
//
//    if(Back2)
//        SDL_FreeSurface(Back2);
//
//    if(New1)
//        SDL_FreeSurface(New1);
//
//    if(New2)
//        SDL_FreeSurface(New2);
//
//    if(Edit1)
//        SDL_FreeSurface(Edit1);
//
//    if(Edit2)
//        SDL_FreeSurface(Edit2);
//
//    if(Save1)
//        SDL_FreeSurface(Save1);
//
//    if(Save2)
//        SDL_FreeSurface(Save2);
//
//    if(LevelEditorTitle)
//        SDL_FreeSurface(LevelEditorTitle);



}

void LoadJoystickSettings()
{
    JoystickSetup->AddDefinition(BUT_SELECT,"Go back / Quit",6,6);
    JoystickSetup->AddDefinition(BUT_R,"Next piece (Leveleditor)",5,5);
    JoystickSetup->AddDefinition(BUT_L,"Previous piece (Leveleditor)",4,4);
    JoystickSetup->AddDefinition(BUT_LEFT,"Left",JOYSTICK_LEFT,JOYSTICK_LEFT);
    JoystickSetup->AddDefinition(BUT_RIGHT,"Right",JOYSTICK_RIGHT,JOYSTICK_RIGHT);
    JoystickSetup->AddDefinition(BUT_DOWN,"Down",JOYSTICK_DOWN,JOYSTICK_DOWN);
    JoystickSetup->AddDefinition(BUT_UP,"Up",JOYSTICK_UP,JOYSTICK_UP);
    JoystickSetup->AddDefinition(BUT_A,"Select / Move piece",0,0);
    JoystickSetup->AddDefinition(BUT_X,"Select / Move piece + Cursor",1,1);
    JoystickSetup->AddDefinition(BUT_START,"Menu",8,8);
    JoystickSetup->LoadCurrentButtonValues("./joystick.def");
}

int main ( int argc, char* argv[] )
{

    SDL_Rect Rect;
    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK);
#if !defined GP2X && !defined CAANOO
	SDL_Surface *Icon;
        if (argc ==2)
        {
            SCREENPOSMULTIPLY = atoi(argv[1]);
            if((SCREENPOSMULTIPLY < 1) || (SCREENPOSMULTIPLY > 10))
                SCREENPOSMULTIPLY = 1;

        }
        WINDOW_WIDTH = 320 *SCREENPOSMULTIPLY,
        WINDOW_HEIGHT = 240*SCREENPOSMULTIPLY,
        TileWidth = 15*SCREENPOSMULTIPLY,
        TileHeight = 15*SCREENPOSMULTIPLY;
#endif
#ifdef GP2X
    if(SDL_GP2X_GetPhysicalScreenSize(&Rect) == 0)
        Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_SWSURFACE );
    else
    {
        TvOutMode = true;
        Screen = SDL_SetVideoMode( 360, 288,16,SDL_SWSURFACE  );
    }
    Buffer = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
#endif
#ifdef WINDOWS
    Icon = IMG_Load("./graphics/dynamate.png");
    SDL_WM_SetIcon(Icon,NULL);
    Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,32, SDL_HWSURFACE );
    SDL_WM_SetCaption( "Dynamate 1.1 - Created By Willems Soft - Orginal game by Jonas Norberg and Bjorn Kalzen", NULL );
    Buffer = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,32,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
#endif
#ifdef CAANOO
    Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_SWSURFACE );
    TvOutMode = false;
    Buffer = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
#endif

    //Buffer = SDL_DisplayFormat(Tmp);
	//SDL_FreeSurface(Tmp);
	if (TvOutMode)
        Buffer1 = SDL_CreateRGBSurface(SDL_HWSURFACE,360,288,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
    else
#if defined GP2X || defined CAANOO
        Buffer1 = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
#else
        Buffer1 = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,32,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
#endif
	//Buffer1 = SDL_DisplayFormat(Tmp);
	//SDL_FreeSurface(Tmp);
    TTF_Init();
    Font = TTF_OpenFont("./font.ttf",12*SCREENPOSMULTIPLY);
    SmallFont = TTF_OpenFont("./font.ttf",11*SCREENPOSMULTIPLY);
    SDL_ShowCursor(SDL_DISABLE);
    Cursor = new CCursor();
#ifdef GP2X
    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,128) < 0)
#elif defined CAANOO
    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,512) < 0)
#else
    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,1024) < 0)
#endif
    {
        GlobalSoundEnabled = false;
        printf("Failed to initialise sound!\n");
        printf("%s\n",Mix_GetError());
    }
    else
    {
        printf("Audio Succesfully initialised!\n");
    }
    LoadSettings();
    SearchForMusic();
    LoadSounds();
    LoadGraphics();
    LoadUnlockData();
    LoadHighScores();
    SearchForLevels();
    JoystickSetup = new CUsbJoystickSetup();
    LoadJoystickSettings();
    SDL_initFramerate(&Fpsman);
    SDL_setFramerate(&Fpsman,30);
    SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,0,0,0));
    SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
    SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
    SDL_Flip(Screen);
    while (GameState != GSQUIT)
    {
        switch(GameState)
        {
            case GSINTRO:
                Intro();
                break;
            case GSINGAMEMENU:
                IngameMenu();
                break;
            case GSGAME:
                Game();
                break;
            case GSDIEDMENU:
                DiedMenu();
                break;
            case GSWINMENU:
                WinMenu();
                break;
            case GSLEVELSELECTMENU:
                LevelSelectMenu();
                break;
            case GSTITLESCREEN:
                TitleScreenMenu();
                break;
            case GSLEVELTYPESELECT:
                LevelTypeMenu();
                break;
            case GSLEVELEDITOR:
                LevelEditor();
                break;
            case GSLEVELEDITORMENU:
                LevelEditorMenu();
                break;
            case GSLEVELEDITORLEVELSELECT:
                LevelEditorLevelSelect();
                break;
            case GSSETUPUSBJOYSTICKBUTTONS:
                SetupUsbJoystickButtons();
                break;
            default:
                break;
        }
    }


    delete Cursor;
    UnLoadSounds();
    Mix_CloseAudio();
#if !defined GP2X && !defined CAANOO
    SDL_FreeSurface(Icon);
#endif
    UnLoadGraphics();
    SaveHighScores();
    SaveUnlockData();
    SaveSettings();
    JoystickSetup->SaveCurrentButtonValues("./joystick.def");
    delete JoystickSetup;
    TTF_CloseFont(SmallFont);
    TTF_CloseFont(Font);
    TTF_Quit();
    SDL_FreeSurface(Screen);
    SDL_FreeSurface(Buffer);
    SDL_FreeSurface(Buffer1);
    SDL_Quit();
    return 0;
}

